pro pds2ncdf,arguments
; Converts profiles of Moments of the pds files produced by the radarserver to the NetCDF 
; format used by dmp2ncdf.
;
; pds2ncdf can be called from the IDL prompt or from a unix shell:
; 
; unix> idl pds2ncdf infile=data/20040602_1000.pds:data/20040602_1200.pds \
;                extract=p1:p2:v1 bdate_begin=DT120000 bdate_end=180000 \
;                outfile=xxx.nc
;
; IDL > pds2ncdf, 'infile=data/20040602_1000.pds extract=p1:p2:sp1 bdate_begin=110000 outfile=xxx.nc'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; extract     An array of tag names which specify which profiles should be extracted.
;             If omitted all profiles which are in the pds file are extracted.
;             If the file is biger than the system memory it is useful to extract
;             only a subset of the profiles. The following 'tags' are allowed: 
;             p1, p2, v1, v2, r1, r2, sp1, sp2.
;
; bdate_begin 
; bdate_end   Specify a date-time range from which data should be extracted.
;             Both must be provided as String in the form yymmddHHMMSS. If leading
;             chars are omitted they are replaced to match the date-time of the
;             first dwell contained in the pds-file.
;
; max_dwells  limits the number of dwells which is loaded per file.
; max_mem     viewpds will stop goging data if this much memory is used.
; outfile     Outputfile
;
;
;
;
; Matthias Bauer-Pfundstein
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

startup_device=!d.name

; Arguments can be provided by two methods:
; - If viewpds is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)
kp = { $
    infile:'',$
    outfile:'',$
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    extract:'P1:V1:R1:P2', $
    bdate_begin:'default', $
    bdate_end:'default', $
    progress_bar:1,$
    calibr:[-2000.,-2000.],$
    do_corrbycal:0,$
    doLowRangeAttenuation:1,$
    max_mem:400000000L, $
    max_headers:5L, $
    max_dwells:70000L $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)

common magic4chs,magic4chs,FzkPdsFmt,syspar
get_noisecom,calida

infiles = strsplit(kp.infile,':',/extract)

extracts = strsplit(kp.extract,':',/extract)

zfiles = n_elements(infiles)
zheaders=kp.max_headers
last_zt = 0L
iheader = 0L
ifile=0L
while ifile lt zfiles do begin
   infile=infiles[ifile]
   gorge_pds_file, infile, ppar, hdr, parr_srvinfo, parr_data, extract=extracts, bdate_begin=kp.bdate_begin, $
        bdate_end=kp.bdate_end,max_dwells=kp.max_dwells,progress_bar=kp.progress_bar,max_mem=kp.max_mem,lun=lun
   if iheader eq 0 then begin
       hdrs=replicate(hdr,zheaders)
       ppars=replicate(ppar,zheaders)
       ztimes = lonarr(zheaders)
   endif
   this_zt = n_elements(parr_srvinfo) * ptr_valid(parr_srvinfo[0])
   if this_zt gt last_zt then begin
       ztimes[iheader] = n_elements(parr_srvinfo) * ptr_valid(parr_srvinfo[0])
       hdrs[iheader] = hdr
       ppars[iheader] = ppar
       if ztimes[iheader] gt 0 then begin
           print,infile,': ',unixtime2bdate((*parr_srvinfo[last_zt]).TIME_T,nice=2,utc=1), $
                   ' - ',unixtime2bdate((*parr_srvinfo[ztimes[iheader]-1]).TIME_T,nice=2,utc=1)
       endif
       ++iheader
   endif
   if lun eq -1000 then ++ifile
   last_zt = this_zt
endwhile
zheaders=iheader

max_ave = max(hdrs[*].ave,min=min_ave)
if max_ave ne min_ave then begin
    print,'Files with different averaging time have been loaded.'
    print,'The ncdf file will represen only the first ppar header.'
endif

max_zmrg = max(hdrs[*].mom_zrg,min=min_zmrg)
if max_zmrg ne min_zmrg then begin
    print,'Files with different number of range gates have been loaded.'
    print,'It is probably not reasonable to pack them to one ncfd file.'
endif

; Open the NetCdf file and write all information to it which occures only once:
ptypes=['g']
extras=['SNRco','SNRcx','Ze']
ncdfid = header2ncdf(kp.outfile,ppars[0],hdrs[0],ptypes=ptypes,extras=extras)

ztime = n_elements(parr_srvinfo) * ptr_valid(parr_srvinfo[0])
if ztime eq 0 then message,'No Data gorged'

zrg=max_zmrg
ldr = fltarr(zrg,/nozero)
snr1 = fltarr(zrg,/nozero)
vinSNRmin = 1.0000001/(hdrs[0].nfft * sqrt(ppars[0].avc))
for iti=0L,ztime-1L do begin
    ; write one time step of the timestamp, txnpow, ...
    srvinfo2ncdf,ncdfid,iti,*parr_srvinfo[iti],hdr.fzk100
    snr1 = (*parr_data[iti]).P1
    ixbad = where(snr1 le vinSNRmin, nixbad)
    if nixbad gt 0 then snr1[ixbad] = !values.f_nan
     
    snr2 = (*parr_data[iti]).P2
    ixbad = where(snr2 le vinSNRmin, nixbad)
    if nixbad gt 0 then snr2[ixbad] = !values.f_nan
     
    ixbad = where(abs((*parr_data[iti]).V1) gt 9000, nixbad)
    if nixbad gt 0 then (*parr_data[iti]).V1[ixbad] = !values.f_nan
    
    ixbad = where(abs((*parr_data[iti]).R1) gt 9000, nixbad)
    if nixbad gt 0 then (*parr_data[iti]).R1[ixbad] = !values.f_nan
    
    ldrgood = (*parr_data[iti]).P1 gt vinSNRmin  and (*parr_data[iti]).P2 gt vinSNRmin
    for irg=0,zrg-1 do begin
        ldr[irg] = ldrgood[irg] ? ((*parr_data[iti]).P2)[irg] / ((*parr_data[iti]).P1)[irg] : !values.f_nan
    endfor

    RadarConstCo = RadarConstant( hdrs[0], *parr_srvinfo[iti], calida, kp.do_corrbycal, kp.doLowRangeAttenuation, SnrCorfa, RadarConst5)
    ; RadarConstCo = RadarConst5 * Hei^2/Hei0^2
    ; Ze[co] = SNR[*] * RadarConstCo * SnrCorfa[*,co]
    ; SNR-Values saved here in to outncdf are not corrected by SnrCorfa as done in mmclx
    ; but the profiles LDRcorr LDRcorr2 and LDRnormal saved here in to outncdf are corrected by SnrCorfa[*,cx]/SnrCorfa[*,co]
    ; LDRg is not corrected
    Ze = RadarConstCo * snr1 * SnrCorfa[*,0]
    
    ; write the profile data to the NetCdf file:
    NCDF_VARPUT, ncdfid, 'SNRg', snr1, count=[zrg,1], offset=[0,iti]
    NCDF_VARPUT, ncdfid, 'SNRco', snr1, count=[zrg,1], offset=[0,iti]
    NCDF_VARPUT, ncdfid, 'SNRcx', snr2, count=[zrg,1], offset=[0,iti]
    NCDF_VARPUT, ncdfid, 'VELg', (*parr_data[iti]).V1, count=[zrg,1], offset=[0,iti]
    NCDF_VARPUT, ncdfid, 'RMSg', (*parr_data[iti]).R1, count=[zrg,1], offset=[0,iti]
    NCDF_VARPUT, ncdfid, 'LDRg', ldr, count=[zrg,1], offset=[0,iti]
    NCDF_VARPUT, ncdfid, 'Ze', Ze, count=[zrg,1], offset=[0,iti]
endfor

ende:
NCDF_CLOSE, ncdfid
heap_free,parr_data
heap_free,parr_srvinfo
exit
end
