;+
; $Id: forward_tkc.pro,v 1.1  $
;
; Abstract:
;	The "Turner-Kneifel-Cadeddu" liquid water absorption model (JTECH 2016).
;    It was built using both laboratory observations (primarily at warm temperatures) and 
;    field data observed by MWRs at multiple frequencies at supercool temperatures. The field
;    data were published in Kneifel et al. JAMC 2014.  The strength of the TKC model is the 
;    use of an optimal estimation framework to determine the empirical coefficients of the 
;    double-Debye model.  A full description of this model is given in 
;
;		Turner, D.D., S. Kneifel, and M.P. Cadeddu, 2016: An improved liquid
;		water absorption model in the microwave for supercooled liquid clouds.
;		J. Atmos. Oceanic Technol., 33(1), pp.33-44, doi:10.1175/JTECH-D-15-0074.1.
;
;  	Note that the model is designed to operate over the frequency range from 0.5 to 500
;    GHz, and temperatures from -40 degC to +50 degC; only for freshwater (no salinity)
;
; 
;
; Authors:
;	Dave Turner, National Severe Storms Laborotory / NOAA
;	Stefan Kneifel, University of Cologne
;	Maria Cadeddu, Argonne National Laboratory
;
; Call:
  function forward_tkc, $		; Returns the liquid mass absorption, in m2 kg-1
  	freq, $				; Input frequency (scalar float), in GHz
	temp, $				; Input cloud temperature (scalar float), in degC
	epsilon=epsilon			; Optional output (complex): the permittivity value
;-

	; Convert the frequency from GHz to Hz
  frq = freq * 1e9

	; Some constants
  cl = 299.792458d6	;speed of light in vacuum
  pi = !dpi

	; Empirical coefficients for the TKC model. The first 4 are a1, b1, c1, and d1, 
	; the next four are a2, b2, c2, and d2, and the last one is tc.
  coef = [8.111e+01, 4.434e-03, 1.302e-13, 6.627e+02, $
  	  2.025e+00, 1.073e-02, 1.012e-14, 6.089e+02, $
	  1.342e+02]

	; This helps to understand how things work below
  a_1 = coef[0]
  b_1 = coef[1]
  c_1 = coef[2]
  d_1 = coef[3]

  a_2 = coef[4]
  b_2 = coef[5]
  c_2 = coef[6]
  d_2 = coef[7]

  t_c = coef[8]


	; Compute the static dielectric permittivity (Eq 6)
  eps_s = 87.9144d - 0.404399d * temp + 9.58726D-4 * temp^2. - 1.32802D-6 * temp^3.

	; Compute the components of the relaxation terms (Eqs 9 and 10)
  		; First Debye component
  delta_1 = a_1 * EXP(-b_1 * temp)
  tau_1   = c_1 * EXP(d_1 / (temp + t_c))
  		; Second Debye component
  delta_2 = a_2 * EXP(-b_2 * temp)
  tau_2   = c_2 * EXP(d_2 / (temp + t_c))

	; Compute the relaxation terms (Eq 7) for the two Debye components
  term1_p1 = (tau_1^2.*delta_1) / (1d + (2d*pi*frq*tau_1)^2.)
  term2_p1 = (tau_2^2.*delta_2) / (1d + (2d*pi*frq*tau_2)^2.)

	; Compute the real permittivitity coefficient (Eq 4)
  eps1 = eps_s - ((2d*pi*frq)^2.)*(term1_p1 + term2_p1) 
	   

	; Compute the relaxation terms (Eq 8) for the two Debye components
  term1_p1 = (tau_1 * delta_1) / (1d + (2d*pi*frq*tau_1)^2.)
  term2_p1 = (tau_2 * delta_2) / (1d + (2d*pi*frq*tau_2)^2.)

	; Compute the imaginary permittivitity coefficient (Eq 5)
  eps2 = 2d*pi*frq * (term1_p1 + term2_p1)
		 
  epsilon = COMPLEX(eps1, eps2)

	; Compute the mass absorption coefficient (Eq 1)
  RE = (epsilon-1)/(epsilon+2)
  alpha = 6.d*!dpi*IMAGINARY(RE)*frq*1d-3/cl

  return,alpha

end
