function interpol_profiles,val,dataquality,hei_in,irg_below,hei_out,reachfactor=reachfactor,nsrcgates=nsrcgates,fstgdrg=fstgdrg,printweight=printweight
; Each profile from the input data of an RHI scan has a differen height resolution due to different elevations.
; This routine interpolates each profile from the input data val to the height grid given in hei_out
; The height of each 
;         val[zrg,  ztime]   Measured Data of any kind (SNR, Zg, VEL ...), denoted as val_in in the comments.
; dataquality[zrg,  ztime]   An optional flag indicating for each data point if it should be used e.g. (Ze[zrg,  ztime] gt -60.0) 
;      hei_in[zrg,  ztime]   Height (not range) of each measurement. 
;   hei_out[zhei_out]        The height grid to which the data will be gridded.
; irg_below[zhei_out, ztime] The number of the gate closest but below each output height 
;   val_out[zhei_out, ztime] The rgidded data returned
; nsrcgates[zhei_out, ztime] Provides the number of gates accounting to each interpolated val_out[zhei_out, ztime] on output
; fstgdrg                    optional parameter: first gate of the input data providing trusted data 
;
; For each interpolation pionts falling into the height range hei_out[ihei_out]+/-reach are used. 
; The keyword parameter reachfactor can be uses to adjust reach to dhei_out*reachfactor, where dhei_out is the spacing between the output heights. 
; In case many gates of the input profile fall into one range hei_out[ihei_out]+/-reach averaging of these input values is most suitable.
; In case two gates of the input profile fall into one range hei_out[ihei_out]+/-reach interpolating them is most suitable.
; In case only one gate of the input profile falls into one range hei_out[ihei_out]+/-reach all that can be done is taking this value.
; interpol_profiles uses a triangle weighted averaging which emulates interpolating in case two input gates fall into the one output gate.
; The base line of the triangle weighting function is the difference in height of the last (irgO) and first (irgA) input point falling into the output gate.
; The interpolation between these two points iA, and IO can be written as:
;     
;      x1=hei_in[irgA]
;      x2=hei_in[irgO]
;      y1=val_in[irgA]
;      y2=val_in[irgO]
;      x=hei_out
;      y=val_out
;      
;      The interpolation is:
;      y = y1 * (x2-x)/(x2-x1)          + y2 * (x-x1)/(x2-x1)
;        = y1 * (x1-x1+x2-x)/(x2-x1)    + y2 * (x2-x2+x-x1)/(x2-x1)   ; by adding 0=x1-x1 on the nominator of the left and 0=x2-x2 on the right fraction
;        = y1 * (x2-x1+x1-x)/(x2-x1)    + y2 * (x2-x1+x-x2)/(x2-x1)   ; by resorting
;        = y1 * (1-abs(x-x1)/(x2-x1)    + y2 * (1-abs(x-x2)/(x2-x1)   ; using x1 < x < x2
; Therefore, the weighting factor can be written as
;      weights[irg]=(1-abs(hei_out-hei_in[irg])/(hei_in[irgO]-hei_in[irgA])     
;
;  Matthias Bauer-Pfundstein
;
    if n_elements(printweight) ne 1 then printweight=0
    zhei_out=n_elements(hei_out)
    dhei_out=hei_out[2]-hei_out[1]
    if n_elements(reachfactor) eq 0 then reachfactor=0.7
    reach = reachfactor * dhei_out
    ztime=n_elements(hei_in[0,*])
    zrg=n_elements(hei_in[*,0]) ; The first and farest gate where added and filled with NaNs
    zrgme=zrg-1
    val_out=fltarr(zhei_out,ztime)
    nsrcgates=intarr(zhei_out,ztime)
    if n_elements(dataquality) le 1 then dataquality=finite(val) 
    if n_elements(fstgdrg) ne 1 then fstgdrg=1

    for iti=0L,ztime-1 do begin
        ; val_out[*,iti] = interpol(val[*,iti],hei_in[0:zrg-1,iti],hei_out)
        
        dhei_in = hei_in[1, iti]-hei_in[0, iti]
        if dhei_in gt 0.0 then begin
            nreach =  long(reach/dhei_in)
            ; For the interpolation of val in height heido[ihei_out] the measured data val[irgA:irgO, iti] 
            for ihei_out=0,zhei_out-1 do begin
                irgA = ((irg_below[ihei_out,iti]-nreach   ) < zrgme) > 0
                irgO = ((irg_below[ihei_out,iti]+nreach+1L) < zrgme) > irgA
                if printweight then print,'hei_out: ' , strgg(hei_out[ihei_out]) + ' input gate number range: '+strgg(irgA)+' ,.. , '+strgg(irgO)
                val4interpol=val[irgA:irgO,iti]
                dataquality4interpol=dataquality[irgA:irgO,iti]
                hei_in4interl=hei_in[irgA:irgO, iti]
                x2mx1=(irgO-irgA)*dhei_in
                weights=1.-abs(hei_in4interl-hei_out[ihei_out])/x2mx1
                if printweight then print,'weights: ',weights
                ixgood4interpol = where(dataquality4interpol,nixgood4interpol)
                nsrcgates[ihei_out,iti]= irgO ge fstgdrg+1 ? 10*total(finite(weights)) : 5 
                ;if ihei_out lt 25 then print,format='("iti=",I4," ihei_out=",I4," irgA=",I4," irgO=",I4," nixgd4interp=",I5," nweights=",I5,$)',iti,ihei_out,irgA,irgO,nixgood4interpol,total(finite(weights))
                if nixgood4interpol ge 1 then begin
                    ;if ihei_out lt 25 then print,format='(" nave=",I5)',total(finite(weights[ixgood4interpol]))
                    totalweight=total(weights[ixgood4interpol])
                    if totalweight/nixgood4interpol gt 0.30 then begin
                        val_out[ihei_out,iti]=total(weights[ixgood4interpol]*val4interpol[ixgood4interpol])/totalweight
                    endif else begin
                        val_out[ihei_out,iti]=!values.f_nan
                    endelse
                endif else begin
                    ;if ihei_out lt 25 then print,''
                    val_out[ihei_out,iti]=!values.f_nan
                endelse
                if printweight then print
            endfor
        endif else begin
            val_out[*,iti]=!values.f_nan
        endelse
    endfor
    return,val_out
end
