;#############################################################################################################
; FILENAME:    get_dwd_profiles.pro
; 
; DESCRIPTION: Contains Function get_dwd_profiles which provides "temperature", "rel. humidity" and
;              "liquid water content" profiles originated from miscellaneos atmosphere messurement devices. 
;              These profiles are read from the DWD-database or calculated approximately.
;              
; IMPLEMENTS:
;  get_dwd_profiles()   : main function. 
;  read_profile()       : sub function. Provides a algorithm for retrievingthe the profiles
;  interpol_profile()   : sub function. Interpolates the profiles
;  get_next_valid()     : sub function. Searches for a valid profile within a time series
;  extend_profile()     : sub function. Updates the time series
;  clearseries()        : sub function. Deletes the whole or particular time series
;  read_from_db()       : sub function. Wrapper function for reading the profiles from the database
;  read_mwp()           : sub function. Reads the mpw-profiles from the database
;  read_raso()          : sub function. Reads the raso-profiles from the database
;  read_model()         : sub function. Reads the the modeled profiles from the database
;  read_inifile()       : sub function. Reads misscelaneous information from ini-files
;  abs_diff()           : sub function. Calculates an absolute arithmetical difference between two scalars
;  db_str2Values()      : sub function. Parses a string from database query into misc. parameter arrays 
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;#############################################################################################################


;************************************************************************************************************
; PROCEDURE:   db_str2Values
; 
; DESCRIPTION: Parses the result string from the database query into miscellaneous arrays with profiles.
;              
;              In dependency on the parameter <proname> te result string is parsed in different ways
;              
;              proname='MWP'
;                 hei  : becomes an array with heights
;                 temp : becomes an array with temperature profiles
;                 n    : contains a number of elements of arrays  
;                 rHum : becomes an array with rel. humidity profiles
;                 aHum : becomes an array with abs. humidity profiles
;                 lwc  : becomes an array with liquid water content profiles
;                 sByte: becomes an array with status bytes for profiles
;                 qFlag: becomes an array with quality flags for profiles
;                 qByte: remains uneffected
;                 
;              proname='RASO'   
;                 hei  : becomes an array with heights
;                 temp : becomes an array with temperature profiles
;                 n    : contains a number of elements of arrays  
;                 rHum : becomes an array with rel. humidity profiles
;                 qByte: becomes an array with quality bytes for profiles
;                 aHum : remains uneffected
;                 lwc  : remains uneffected
;                 sByte: remains uneffected
;                 qFlag: remains uneffected
;                 
;              proname='MODEL'   
;                 hei  : becomes an array with heights
;                 temp : becomes an array with temperature profiles
;                 n    : contains a number of elements of arrays  
;                 rHum : remains uneffected
;                 qByte: remains uneffected
;                 aHum : remains uneffected
;                 lwc  : remains uneffected
;                 sByte: remains uneffected
;                 qFlag: remains uneffected
;                 
;                 
; PARAMETERS:
;  str         uneffected. result string from the database query
;  hei         effected. becomes an array with heights
;  temp        effected. becomes an array with temperature profiles
;  n           effected. contains a number of elements of arrays  
;  rHum        effected. becomes an array with rel. humidity profiles
;  aHum        effected. becomes an array with abs. humidity profiles
;  lwc         effected. becomes an array with liquid water content profiles
;  sByte       effected. becomes an array with status bytes for profiles
;  qFlag       effected. becomes an array with quality flags for profiles  
;  qByte       effected. becomes an array with quality bytes for profiles
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
pro db_str2Values, str,  hei, temp, n, proname=proname, qByte=qByte, rHum=rHum, aHum=aHum, lwc=lwc, qFlag=qFlag, sByte=sByte
   n=n_elements(str)
   if N_ELEMENTS(proname) ne 0 then proname=STRUPCASE(proname) else proname='MWP'
   if n gt 30 then begin
      case proname of 
      'RASO' :begin
         hei=fltarr(n) & temp=fltarr(n) & rHum=fltarr(n) & qByte=bytarr(n)
         for i=0, n-1 do begin
            ht=strsplit(str[i],/extr) & hei[i]=float(ht[0]) & temp[i]=float(ht[1]) & rHum[i]=float(ht[2]) & qByte[i]=byte(ht[3]) 
         endfor 
      end
      'MWP'  :begin
         hei=fltarr(n) & temp=fltarr(n) & rHum=fltarr(n) & aHum=fltarr(n)
         lwc=fltarr(n)  & qFlag=bytarr(n) & sByte=bytarr(n)
         for i=0,n-1 do begin
            ht=strsplit(str[i],/extr) 
            hei[i]=float(ht[0]) & temp[i]=float(ht[1]) & rHum[i]=float(ht[2])
            aHum[i]=float(ht[3]) & lwc[i]=float(ht[4]) & sByte[i]=byte(ht[6])
            case strupcase(ht[5]) of
               'G':  qFlag[i]=1
               'D':  qFlag[i]=2
               'B':  qFlag[i]=3
               else: qFlag[i]=2
            endcase
         endfor        
      end
      'MODEL':begin
         hei=fltarr(n) & temp=fltarr(n)
         for i=0,n-1 do begin
            ht=float(strsplit(str[i],/extr)) & hei[i]=ht[0] & temp[i]=ht[1]
         endfor 
      end
      endcase
   endif else n=0
end

;************************************************************************************************************
;debug function 
;************************************************************************************************************
pro strarr2ht_debug,str,hei,temp,n
    n=n_elements(str)
    if n gt 5 then begin; if n gt 30 then begin
        hei=fltarr(n) & temp=fltarr(n)
        for i=0,n-1 do begin
            ht=float(strsplit(str[i],/extr)) & hei[i]=ht[0] & temp[i]=ht[1]
        endfor
    endif else n=0
end

;************************************************************************************************************
;debug function 
;************************************************************************************************************
pro strarr2ht_debug_plus,str,hei,temp,rel_hum, hum,qmess, status, n
    n=n_elements(str)
    if n gt 5 then begin
        hei=fltarr(n) & temp=fltarr(n) & rel_hum=fltarr(n) & hum=fltarr(n) & qmess=strarr(n) & status=intarr(n) 
        for i=0,n-1 do begin
            ht=strsplit(str[i],/extr) 
            hei[i]=float(ht[0]) 
            temp[i]=float(ht[1]) 
            rel_hum[i]=float(ht[2]) 
            hum[i]=float(ht[3]) 
            qmess[i]=ht[4] 
            status[i]=fix(ht[5])
        endfor
    endif else n=0
end

;************************************************************************************************************
; FUNCTION:    abs_diff
; 
; DESCRIPTION: Returns an absolute arithmetical difference between to scalar values <time1> and <time2> 
;              
; PARAMETERS:
;  time1, time2: two scalar values between which the absolute arithmetical difference is to be calculated
;  
; RESULT OF FUNCTION: Absolute arithmetical difference between to scalar values <time1> and <time2> 
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function abs_diff, time1, time2
   return, max([time1, time2])-min([time1, time2])
end

;************************************************************************************************************
; FUNCTION:    read_inifile
; 
; DESCRIPTION: Reads list of prefered retrieval sources for reading the profiles and information about 
;              how handle them from and ini-file 
;              
;              If no parameter <infile> is given or default ini-file couldno be found a pick-file
;              dialog is shown.
;              
;              If the pick-file dialog was canceled or selected ini-file ist not valid, the function returns
;              default values.
;              
; PARAMETERS:
;  infile:     effected, optional. Should contain a full path with filename to a valid ini-file
;              If this parameter was given it contains afer execution of the function the full
;              path and file name of the ini-file, which was used or an empty string if no ini-file
;              was found.
;  
; RESULT OF FUNCTION: List of prefered retrieval sources for reading the profiles and information about 
;              how handle them. This List consists from array of structures with 3 elemets. The index number
;              of these elements gives a fall back ordering which is used by the algorithm if the
;              first kind of retirieval from this list contains no data. The array elements are structure of
;              stru={
;                 proname: string; Name of Profile. Options are: 'MWP' 'RASO' or 'MODEL'
;                 max_interpol_hr: unsigned long; max. number of hours between two data sets 
;                                  in which the interpolation may return acceptable values
;                                  i.e.: (mwp) max_interpol_hr=6, (raso) max_interpol_hr=12
;                                        (model) max_interpol_hr=0
;                 read_step_minutes: unsigned long; defines an intervall in minutes between 
;                                  data sets for a data base query. It is usefull to know how often the
;                                  data is stored into the data base. i.e. timeseries from the mwp device are 
;                                  stored every 10 minutes. 
;              }         
;               
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function read_inifile, infile=infile
   common retrievalmodes, ini_file
   if strupcase(getenv('HOSTNAME')) eq 'NEANDER' then begin
      ; Set this file for development and debuging on NEANDER-PC
      default_file='/home/alex/work/dwd/m36s/idl/profile_retrieval/retrievals.ini'
   endif else begin
      ; Set this file for running on DWD site
      default_file='/home/data/metek/m36s/local/idl/dwd1/profile_retrieval/retrievals.ini'
      
      ; Set this file for running on Auswertung site
      ; default_file='/home/data/metek/cloudradar/idl/profile_retrieval/retrievals.ini'
   endelse
   
   count=0
   file=-1
   retr={proname:'', max_interpol_hr:0UL, read_step_minutes:0UL}
   default_res=replicate(retr, 3)
   default_res[0].proname='MPW'   & default_res[0].max_interpol_hr=6    & default_res[0].read_step_minutes=10UL
   default_res[1].proname='RASO'  & default_res[1].max_interpol_hr=12   & default_res[1].read_step_minutes=360UL
   default_res[2].proname='MODEL' & default_res[2].max_interpol_hr=0    & default_res[2].read_step_minutes=0UL
   
   if N_ELEMENTS(infile) ne 0 then begin
      file = FILE_SEARCH(infile, count=count)
      if count eq 0 then print, 'File not found: '+infile
   endif 
   
   if count eq 0 then begin
      if (N_ELEMENTS(ini_file) eq 0) || ((N_ELEMENTS(ini_file) eq 1)&&(ini_file eq -1)) then begin
         file = FILE_SEARCH(default_file, count=count)
         if count eq 0 then begin
            filter=['*.ini']
            dispname='Select an ini-file for the retrieval art'
            title=dispname
            file=DIALOG_PICKFILE(filter=filter, DISPLAY_NAME=disname, title=title, /read, /FIX_FILTER)            
         endif
         if file ne '' then ini_file=file else begin
            print, 'No ini-file selected or default ini-file not found'
            infile=''
            return, default_res
         endelse
      endif   
   endif else begin
      ini_file=file
   endelse
   infile=ini_file
   openr, unit, ini_file, /GET_LUN
   str='' & ix=0 & res=retr & interpol_ix=0 & proname_ix=0 & step_ix=0
   while (not EOF(unit)) do begin
      readf, unit, str
      pos=strpos(str, ';')
      if pos ge 0 then begin
         str=STRMID(str,0,pos)
      endif
      
      if (strPos(str, '[') gt -1) && (strPos(str, ']') gt -1)  then begin
         id = fix((STRSPLIT(str, '[]', /EXTRACT))[0])
         if N_ELEMENTS(idarr) eq 0 then idarr=id else begin
            idarr=[idarr,id]
            ix++
            res=[res, retr]
         endelse         
      endif
      
      if strPos(str, 'proname') gt -1 then begin
         res[ix].proname=STRUPCASE((STRSPLIT(str, '=', /EXTRACT))[1])
         proname_ix++
      endif
      if strPos(str, 'max_interpol_hr') gt -1 then begin
         res[ix].max_interpol_hr=(STRSPLIT(str, '=', /EXTRACT))[1]
         interpol_ix++
      end         
      if strPos(str, 'read_step_minutes') gt -1 then begin
         res[ix].read_step_minutes=(STRSPLIT(str, '=', /EXTRACT))[1]
         step_ix++
      end
   endwhile
   
   if (proname_ix eq N_ELEMENTS(idarr)) &&  (proname_ix eq interpol_ix) && (step_ix eq interpol_ix) then begin
      sorted=sort(idarr)
      res=res[sorted]   
   endif else res=default_res   
   
   if N_ELEMENTS(unit) ne 0 then FREE_LUN, unit
   return, res
end

;************************************************************************************************************
; FUNCTION:    read_model
; 
; DESCRIPTION: Reads modeled profiles at requested timestamp <utime> from the DWD-database and returnes a 
;              structure as result (see description RESULT OF FUNCTION) 
;              
; USES (description of importent routines):
;  unixtime2bdate()           Converts unix time (seconds since 1970..) into a human readable string
;  
;  /home/data/bin/readFromDb: a spawned wrapper shell script for quering the dwd-database.
;                             Resides on the auswertung-PC (see CONDITIONS)
;                             
;  /home/data/bin/get_mwp.pl: a spawned pearl scripts that reads profiles from the dwd-database.
;                             Resides on the auswertung-PC (see CONDITIONS)
;  
;  db_str2Values()            parses the result string from the database query into arrays with profiles
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  utime:      uneffected. Gives a utc unix time stamp (seconds since the begin of year 1970) 
;              at which profiles should be read.
; 
;  hei:        uneffected. Gives a range (array) of heights for the requested profiles
;  
;  agl2asl:    uneffected, optional. Gives the height above the sea level of the location site;  
; 
; RESULT OF FUNCTION: A structure with temperature profile is returned:
;              stru={
;                 temp  : Temperature profile within range of heights
;              }; 
;              
;              In case no data could be read function returns -1
;               
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function read_model, utime, hei, agl2asl=agl2asl
   common dwd_model, modelTime, heiModel, tempModel, nModel
   if strupcase(getenv('HOSTANME')) ne 'NEANDER' then begin 
      if N_ELEMENTS(modelTime) eq 0 then modelTime=-1
      host=getenv('HOSTNAME')
      if host ne 'plgfe3l004.dwd.de' then doitat='ssh auswertung ' else doitat=''
      nowis = ulong(systime(/sec,/utc))
      H6=3600UL*6UL
      H1 = 3600UL
      modelTimex = ( (utime + 1790UL) < (nowis-0UL) ) / H1 * H1
      if modelTimex ne modelTime then begin
         modelTime=modelTimex
         modelTimeStr = unixtime2bdate(modelTime,nice=5, /UTC)
         print,'loading modeldata '+unixtime2bdate(modelTime,nice=2, /UTC)
         spawn,doitat+'/home/data/bin/readFromDb /home/data/bin/modeldata1 '+modelTimeStr,result
         if n_elements(result) gt 2 then begin
            db_str2Values,result[1:*],heiModel,tempModel,nModel, PRONAME='MODEL'
         endif else nModel=0
      endif 
      if n_elements(nModel) ne 1 then  nModel=0
      if nModel gt 0 then begin
         temp = interpol(tempModel,heiModel,hei+agl2asl,/quadratic)
         res={temp:temp}
         return,res
      endif
   endif
   return, -1
end

;************************************************************************************************************
;debug function 
;************************************************************************************************************
function read_raso_debug, utime, doitat
   H6=3600UL*6UL
   ttime=((utime + h6-1L) ) / h6 *h6 
   scriptStr='/home/alex/work/dwd/m36s/bin/get_raso_at '
   
   timeStr = unixtime2bdate(ttime,nice=4, /UTC)
   print,'loading RASO-data at '+unixtime2bdate(ttime,nice=2, /UTC)
   spawn, scriptStr + timeStr,db_result
   
   strarr2ht_debug,db_result,db_hei,db_temp,db_n
   
   ; is there a profile loaded?
   if ((size(db_result, /DIMENSIONS))[0] lt 2) or (db_n eq 0) then begin
      print, 'No RASO-profile found at '+unixtime2bdate(ttime,nice=2, /UTC)
      print, 'REASON: '+db_result
      res={time:ttime, data:ptr_new()}
   endif else begin
      res={ time:ttime, $
            data:ptr_new({hei:db_hei, temp:db_temp, rHum:db_temp }) $
          }
   endelse
   return, res
end

;************************************************************************************************************
; FUNCTION:    read_mwp
; 
; DESCRIPTION: Reads RASO-profiles at requested timestamp <utime> from the DWD-database and returnes a 
;              structure with profiles and timestamp (see description RESULT OF FUNCTION) 
;              
;              Due to the fact that RASO profiles are available for every 6 hours in the database
;              the function rasterizes the given timestamp to the next multiple of 6 hours of day.
;              i.e. the requested timestamp is 1223813550 (12:12:30 12.10.2008) it will be rasterized 
;              to 1223834400 (18:00:00 12.10.2008)
;              
; USES (description of importent routines):
;  unixtime2bdate()           Converts unix time (seconds since 1970..) into a human readable string
;  
;  /home/data/bin/readFromDb: a spawned wrapper shell script for quering the dwd-database.
;                             Resides on the auswertung-PC (see CONDITIONS)
;                             
;  /home/data/bin/get_mwp.pl: a spawned pearl scripts that reads profiles from the dwd-database.
;                             Resides on the auswertung-PC (see CONDITIONS)
;  
;  db_str2Values()            parses the result string from the database query into arrays with profiles
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  utime       uneffected. Gives a unix timestamp at which a new profile is to be queried from the database
;  
;  doitat:     uneffected. If this function runs on the PC of DWD cloudradar this parameter schould a string
;              containing 'ssh auswertung ' string, so that database query can be spawned via ssh connetcion
;              to the auswertung-PC. In other cases (see CONDITIONS) this parameter should be an empty string. 
; 
; RESULT OF FUNCTION: A structure with timesereis is returned:
;              stru={
;                 time: array of timestamps
;                 data: array with pointers to profiles
;              }; 
;              
;              One element with the index N of array stru.data points to a structure containing a profile which 
;              corresponds to the timestamp with the same index N of the array stru.time.
;              
;              *(stru.data[N])={
;                 hei   : Range of heights for this profile
;                 temp  : Temperature profile within range of heights
;                 rHum  : Rel humidity profile within range of heights
;                 lwc   : Liquid water content profile within range of heightsAvailable just at MWP 
;              }
;              
;              In case that no data was read, stru.data contanes empty pointers
;               
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function read_raso, utime, doitat
   if strupcase(getenv('HOSTNAME')) eq 'NEANDER' then return, read_raso_debug(utime, doitat)
   H6=3600UL*6UL
   ;ttime=((utime + h6-1L) ) / h6 *h6 
   ttime=((utime) ) / h6 *h6 
   timeStr=unixtime2bdate(ttime, nice=5, /UTC)
   print,'loading RASO-data at '+unixtime2bdate(ttime,nice=2, /UTC)
   spawn,doitat+'/home/data/bin/readFromDb /home/data/bin/get_raso.pl '+timeStr, result
   db_str2Values, result, hei, temp, n, rHum=rHum, qByte=qByte, proname='RASO'
   if N_ELEMENTS(n) ne 1 then n=0
   if n gt 0 then begin
      res={ time:ttime, $
            data:ptr_new({hei:hei, temp:temp, rHum:rHum })$
          }
   endif else begin
      print, 'an empty raso-profile was read'
      res={time:ttime, data:ptr_new()}
   endelse
   return,res
end

;************************************************************************************************************
;debug function 
;************************************************************************************************************
function read_mwp_debug, utime, doitat
    M1 =60UL
   M10=M1*10UL
   ttime=((utime + M10-1L) ) / M10 *M10
   
   scriptStr='/home/alex/work/dwd/m36s/bin/get_mwp_at_plus '
   timeStr = unixtime2bdate(ttime,nice=4, /UTC)
   print,'loading MWP-data at '+unixtime2bdate(ttime,nice=2, /UTC)
   spawn, scriptStr + timeStr,db_result
   
   strarr2ht_debug_plus, db_result,db_hei,db_temp, rel_hum, hum, qmess, status, db_n
   
   ; is there a profile loaded?
   if ((size(db_result, /DIMENSIONS))[0] lt 2) or (db_n eq 0) then begin
      print, 'No MWP-profile found at '+unixtime2bdate(utime,nice=2, /UTC)
      print, 'REASON: '+db_result
      res={time:ttime, data:ptr_new()}
   endif else begin
      res={ time:ttime, $
            data:ptr_new({hei:db_hei, temp:db_temp, rHum:rel_Hum, lwc:hum  }) $
          }
   endelse
   return, res
end

;************************************************************************************************************
; FUNCTION:    read_mwp
; 
; DESCRIPTION: Reads mwp-profiles at requested timestamp <utime> from the DWD-database and returnes a 
;              structure with profiles and timestamp (see description RESULT OF FUNCTION) 
;              
;              Due to the fact that mwp profiles are available for every 10 minutes in the database
;              the function rasterizes the given timestamp to the next multiple of 10 minutes of hour.
;              i.e. the requested timestamp is 1223813550 (12:12:30 12.10.2008) it will be rasterized 
;              to 1223814000 (12:20:00 12.10.2008)
;              
;              The function evalutes also the quality- and status-flags of read profiles and skips
;              dubious profiles and profiles which were recorded during a havy rain.
;               
;              
; USES (description of importent routines):
;  unixtime2bdate()           Converts unix time (seconds since 1970..) into a human readable string
;  
;  /home/data/bin/readFromDb: a spawned wrapper shell script for quering the dwd-database.
;                             Resides on the auswertung-PC (see CONDITIONS)
;                             
;  /home/data/bin/get_mwp.pl: a spawned pearl scripts that reads profiles from the dwd-database.
;                             Resides on the auswertung-PC (see CONDITIONS)
;  
;  db_str2Values()            parses the result string from the database query into arrays with profiles
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  utime       uneffected. Gives a unix timestamp at which a new profile is to be queried from the database
;  
;  doitat:     uneffected. If this function runs on the PC of DWD cloudradar this parameter schould a string
;              containing 'ssh auswertung ' string, so that database query can be spawned via ssh connetcion
;              to the auswertung-PC. In other cases (see CONDITIONS) this parameter should be an empty string. 
; 
; RESULT OF FUNCTION: A structure with timesereis is returned:
;              stru={
;                 time: array of timestamps
;                 data: array with pointers to profiles
;              }; 
;              
;              One element with the index N of array stru.data points to a structure containing a profile which 
;              corresponds to the timestamp with the same index N of the array stru.time.
;              
;              *(stru.data[N])={
;                 hei   : Range of heights for this profile
;                 temp  : Temperature profile within range of heights
;                 rHum  : Rel humidity profile within range of heights
;                 lwc   : Liquid water content profile within range of heightsAvailable just at MWP 
;              }
;              
;              In case that no data was read or dubious or rainy data was skipped, 
;              stru.data contanes empty pointers
;               
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function read_mwp, utime, doitat
   if strupcase(getenv('HOSTNAME')) eq 'NEANDER' then return, read_mwp_debug(utime, doitat)
   M1 =60UL
   M10=M1*10UL
   ttime=((utime + M10-1L) ) / M10 *M10
   timeStr=unixtime2bdate(ttime, nice=4, /UTC)
   print,'loading MWP-data at '+unixtime2bdate(ttime,nice=2, /UTC)
   spawn,doitat+'/home/data/bin/readFromDb /home/data/bin/get_mwp.pl '+timeStr, result
   db_str2Values, result, hei, temp, n, rHum=rHum, aHum=aHum, lwc=lwc, qFlag=qFlag, sByte=sByte , proname='MWP'
   if N_ELEMENTS(n) ne 1 then n=0
   if n gt 0 then begin   
      ; skip dubios and bad profile data
      tmp=where(qFlag eq 1, found)
      if found gt 0 then begin
         if n ne found then begin
            print, 'skipping dudios mwp-profile'
            return, {time:ttime, data:ptr_new()}
         endif
         hei=hei[tmp] & temp=temp[tmp] & rHum=rHum[tmp] & aHum=aHum[tmp] & lwc=lwc[tmp] & qFlag=qFlag[tmp] & sByte=sByte[tmp] 
      endif
      
      ;skip rainy profile data
      tmp=where(sByte eq 1, found)
      if (min(rHum) le 0.00) && (min(aHum) le 0.00) && (found gt 0) then begin
         print, 'skipping rainy mwp_profile'
         res={time:ttime, data:ptr_new()}
      endif else begin 
         res={ time:ttime, $
               data:ptr_new({hei:hei, temp:temp, rHum:rHum, lwc:lwc  }) $
          }
      endelse
   endif else begin
      print, 'an empty mwp-profile was read'
      res={time:ttime, data:ptr_new()}
   endelse
   return,res
end

;************************************************************************************************************
; FUNCTION:    read_from_db
; 
; DESCRIPTION: Delegates in dependancy on the parameter <utime> requests on corresponding function for
;              reading profiles at given timestamp <utime> 
;              
; USES (description of importent routines):
;  read_raso() reads raso-profiles from the dwd-database
;  read_mwp()  reads mwp-profiles from the dwd-database
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  utime       uneffected. Gives a timestamp at which a new profile is to be queried from the database
;  
;  proname     uneffected, optional. Gives the name ( MWP or RASO) of the profile which is to be queried 
;              from the database
; 
; RESULT OF FUNCTION: a structure with timesereis is returned:
;              stru={
;                 time: array of timestamps
;                 data: array with pointers to profiles
;              }; 
;              
;              One element with the index N of array stru.data points to a structure containing a profile which 
;              corresponds to the timestamp with the same index N of the array stru.time.
;              
;              *(stru.data[N])={
;                 hei   : Range of heights for this profile
;                 temp  : Temperature profile within range of heights
;                 rHum  : Rel humidity profile within range of heights
;                 lwc   : (Just mwp) Liquid water content profile within range of heightsAvailable just at MWP 
;              }
;               
;              In case that no data was read, stru.data contanes empty pointers
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function read_from_db, utime, proname=proname
   host=getenv('HOSTNAME')
   if host ne 'plgfe3l004.dwd.de' then doitat='ssh auswertung ' else doitat=''
   if N_ELEMENTS(proname) ne 0 then proname=STRUPCASE(proname) else proname='MWP'
   if proname eq 'RASO'then db_res=read_raso(utime, doitat) else db_res=read_mwp(utime, doitat)
   return, db_res
end


;************************************************************************************************************
; FUNCTION:    clearseries
; 
; DESCRIPTION: The whole timeseries or just withing the given range of time will be deleted. The allocated
;              memory for the ponter array proseries.data[range] which is to be deleted will be freed.
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS: 
;  proseries   effected. Contains timeseries with profiles which are to be deleted
;  
;  fromTo      uneffected, optional. Should contain an array with two elements (timestamps) which give
;              an intervall boundary for the time range in which profiles are to be deleted.
;              
;              If this parameter is not given the whole timeseries will be deleted
;              
; RESULT OF FUNCTION: In case the whole timeseries were deleted the function returns -1. In other
;              cases a structure with timesereis is returned:
;              stru={
;                 time: array of timestamps
;                 data: array with pointers to profiles
;              }; 
;              
;              One element with the index N of array stru.data points to a structure containing a profile which 
;              corresponds to the timestamp with the same index N of the array stru.time.
;              
;              *(stru.data[N])={
;                 hei   : Range of heights for this profile
;                 temp  : Temperature profile within range of heights
;                 rHum  : Rel humidity profile within range of heights
;                 lwc   : (Just mwp) Liquid water content profile within range of heightsAvailable just at MWP 
;              }
;               
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function clearseries, profiles, fromTo=fromTo
   if N_ELEMENTS(fromTo) eq 0  then begin
      profiles.time=-1
      HEAP_FREE, profiles.data
      profiles=-1
      res=-1
   endif else begin
      if N_ELEMENTS(fromTo) eq 2 then begin
         ix1=where(profiles.time ge fromTo[0], count1)
         ix2=where(profiles.time le fromTo[1], count2)
         if fromTo[0] le fromTo[1] then begin
            if count1 gt 0 && count2 gt 0 then begin
               ix=where(ix1 le max(ix2))
               sv1=where(where(profiles.time gt 0UL) lt min(ix), cnt_sv1)
               sv2=where(where(profiles.time gt 0UL) gt max(ix), cnt_sv2)
               if cnt_sv1 gt 0 then sv=sv1
               if cnt_sv2 gt 0 then begin
                  if N_ELEMENTS(sv) ne 0 then sv=[sv, sv2] else sv=sv2
               endif   
            endif
         endif else begin
            if count1 gt 0 then ix=ix1
            if count2 gt 0 then begin
               if N_ELEMENTS(ix) ne 0 then ix=[ix,ix2] else ix=ix2                
            endif
            if N_ELEMENTS(ix) ne 0 then begin
               sv1=where(where(profiles.time gt 0UL) lt min(ix), cnt_sv1)
               sv2=where(where(profiles.time gt 0UL) gt max(ix), cnt_sv2)
               if cnt_sv1 gt 0 then sv=sv1
               if cnt_sv2 gt 0 then begin
                  if N_ELEMENTS(sv) ne 0 then sv=[sv, sv2] else sv=sv2
               endif
            endif
         endelse
         if (N_ELEMENTS(ix) ne 0 || (N_ELEMENTS(ix) eq 1 && (ix ne -1))) then begin 
            tmp=profiles.data[ix]
            if N_ELEMENTS(sv) ne 0 then res={data:profiles.data[sv],time:profiles.time[sv]} else res=-1
            HEAP_FREE, tmp
         endif   
      endif
   endelse   
   return, res
end

;************************************************************************************************************
; FUNCTION:    extend_profile
; 
; DESCRIPTION: This function updates (fills, prepends, overwrites or appends) the given as parameter 
;              timeseries <proseries> with new profiles which are read from the DWD-database.
;              
;              Filling the timeseries: expected parameters are: <proseries=-1>, <retrieval>, <nHours>, <starttime>
;              Beginning from starttime until starttime+nHours new profiles are read from the DWD-database
;              (queried for every timestamp of retrieval.read_step_minutes) and inserted into the timeseries
;              
;              Prepending new profiles: expected parameters are: <proseries>, <retrieval>, <direction=0>, <nHours>
;              Beginning with the oldest timestamp of proseries - nHours until the oldest timestamp of 
;              the timestamp new profiles are read from the DWD-database (queried for every timestamp of 
;              retrieval.read_step_minutes) and prepended in front of the oldest profile of the timeseries.
;
;              Appending new profiles: expected parameters are: <proseries>, <retrieval>, <direction=1>, <nHours>
;              Beginning from the youngest timestamp of proseries until the youngest timestamp of 
;              the timestamp + nHours new profiles are read from the DWD-database (queried for every 
;              timestamp of retrieval.read_step_minutes) and appended behind the youngest profile of the 
;              timeseries.
;              
;              The algorithm determines also that during prepending or appending profiles may be read 
;              from the DWD-database which already exist in the timeseries. In this case new profiles
;              will overwrite the stored profiles. 
;              
; USES (description of importent routines):
;  read_from_db() reads profiles from the DWD-database at given timestamp 
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  proseries:  effected. Contains timeseries with profiles which are to be updated with new profiles
;              read from the DWD-database. If this parameter is given with value of -1 this means 
;              that the tieseries is empty nad is to be filled. In this case the optional parameter
;              <starttime> is expected 
;
;  retrieval:  uneffected. contains a structure with information about the timeseries 
;              wich are to be updated
;              stru={
;                 proname: string; Name of Profile. Options are: 'MWP' 'RASO' or 'MODEL'
;                 max_interpol_hr: unsigned long; max. number of hours between two data sets 
;                                  in which the interpolation may return acceptable values
;                                  i.e.: (mwp) max_interpol_hr=6, (raso) max_interpol_hr=12
;                                        (model) max_interpol_hr=0
;                 read_step_minutes: unsigned long; defines an intervall in minutes between 
;                                  data sets for a data base query. It is usefull to know how often the
;                                  data is stored into the data base. i.e. timeseries from the mwp device are 
;                                  stored every 10 minutes. 
;              }        
;    
;  direction:  uneffected, optional; deteremines how the timeseries are to be updated
;                            
;              0: new profiles read from the DWD-database within time intervall of
;                 [OTP-nHours..OTP] are prepended
;              1: new profiles read from the DWD-database within time intervall of
;                 [YTP..YTP+nHours] are appended    
;     
;                 OTP = The profile of the timeseries with the oldest timestamp
;                 YTP = The profile of the timeseries with the youngest timestamp
;                 
;              If this parameter is not given it is set to default value of 1
;              
;  nHours:     uneffected, optional; gives number of hours (time intervall) for which
;              new profiles are to be read from the database. 
;              
;  starttime:  uneffected, optional; strongly required if parameter <proseries> is given as -1
;              Gives a begining of the time intervall fro filling the timeseries
; 
; RESULT OF FUNCTION: In case no data can be read from database to fill the timeseries or
;              wrong combination of parameters is given the function returns -1. In other
;              cases a structure with updated timesereis is returned:
;              stru={
;                 time: array of timestamps
;                 data: array with pointers to profiles
;              }; 
;              
;              One element with the index N of array stru.data points to a structure containing a profile which 
;              corresponds to the timestamp with the same index N of the array stru.time.
;              
;              *(stru.data[N])={
;                 hei   : Range of heights for this profile
;                 temp  : Temperature profile within range of heights
;                 rHum  : Rel humidity profile within range of heights
;                 lwc   : (Just mwp) Liquid water content profile within range of heightsAvailable just at MWP 
;              }
;               
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function extend_profile,  proseries, retrieval, direction=direction, nHours=nHours, starttime=starttime
   ONEHOUR=3600UL

  
   if N_ELEMENTS(direction) ne 0 then dir=direction else dir=1
   if N_ELEMENTS(nHours) ne 0 then intervall=nHours*ONEHOUR else intervall=ONEHOUR
   if (size(proseries))[0] eq 0 && proseries eq -1 then begin
      if N_ELEMENTS(starttime) ne 0 then begin
         utime=starttime
      endif else begin
         print, 'extend_profile: Expecting a valid profile or starttime argument'
         return, -1
      endelse
   endif else begin
      if dir gt 0 then utime=proseries.time[N_ELEMENTS(proseries.time)-1]+ONEHOUR else utime=proseries.time[0]-intervall
      starttime=utime 
   endelse
   
   while abs_diff(starttime,utime) lt intervall do begin
      onepro=read_from_db(utime, proname=retrieval.proname)
      ; TODO Check here if data is valid 
      if (size(onepro))[0] ne 0 || onepro ne -1 then begin
         if (size(proseries))[0] eq 0 && proseries eq -1 then begin
            proseries=onepro 
         endif else begin
            
            ol_right=where(proseries.time ge min(onepro.time), count_r)
            ol_left =where(proseries.time le max(onepro.time), count_l)
            
            if count_r eq N_ELEMENTS(proseries.time) && count_l eq N_ELEMENTS(proseries.time) then begin
               ; Schwere entscheidung. Was ist wenn porseries profile enthaelt, nicht in onepro auftreten?
               proseries=onepro
            endif else begin  
               if count_r eq N_ELEMENTS(proseries.time) then count_r=0
               if count_l eq N_ELEMENTS(proseries.time) then count_l=0
               
               if count_r eq 0 && count_l eq 0 then begin
                  ;both series do not overlap 
                  if min(onepro.time) gt max(proseries.time) then begin 
                     proseries={ time:[proseries.time, onepro.time], data:[proseries.data, onepro.data]}
                  endif else begin  
                     proseries={ time:[onepro.time, proseries.time], data:[onepro.data, proseries.data]}
                  endelse               
               endif else begin 
                  if count_r ne 0 && count_l eq 0 then begin
                     ; onepro overlaps proseries on the right (onepro has younger series)
                     ix1=where(onepro.time gt max(proseries.time), count1) ;-> store this 
                     ix2=where(onepro.time le max(proseries.time), count2) ;-> delete this 
                     if count1 gt 0 then begin
                        proseries={time:[proseries.time, onepro.time[ix1]], data:[proseries.data, onepro.data[ix1]]}
                     endif
                     if count2 ne 0 then heap_free, onepro.data[ix2]                        
                                 
                  endif else begin
                     if count_r eq 0 && count_l ne 0 then begin
                        ; onerpo overlapps proseries on the left (proseries has younger series)
                        ix1=where(onepro.time lt min(proseries.time), count1) ;-> store this 
                        ix2=where(onepro.time ge min(proseries.time), count2) ;-> delete this 
                        if count1 gt 0 then begin
                           proseries={time:[onepro.time[ix1], proseries.time], data:[onepro.data[ix1],proseries.time]}
                           if count2 ne 0 then heap_free, onepro.data[ix2]
                         endif
                     endif else begin
                        ; proseries overlaps onepro on both sides (proseries has older an younger series)
                        ix1=max(where(proseries.time lt min(onepro.time), count1))
                        ix2=min(where(proseries.time gt max(onepro.time), count1))
                        ix3=N_ELEMENTS(proseries.time)-1
                        tmppro={$
                           time:[proseries.time[0:ix1], onepro.time, proseries.time[ix2:ix3]],$
                           data:[proseries.data[0:ix1], onepro.data, proseries.data[ix2:ix3]] $
                        }
                        proseries=tmppro                                  
                     endelse
                  endelse
               endelse
            endelse
         endelse 
      endif else begin
         ; an empty profile was read
         if (size(proseries))[0] eq 0 && proseries eq -1 then begin
            proseries={time:[utime], data:[ptr_new()]} 
         endif else begin
            ix1=max(where(proseries.time lt utime, count1))
            ix2=min(where(proseries.time gt utime, count2))
            ix3=N_ELEMENTS(proseries.time)-1
            if count1 gt 0 && count2 gt 0 then begin
               proseries={ time:[proseries.time[0:ix1], utime, proseries.time[ix2:ix3]],$
                           data:[proseries.data[0:ix1], ptr_new(), proseries.data[ix2:ix3]] }
            endif else begin
               if count1 gt 0 then begin
                  proseries={time:[proseries.time, utime], data:[proseries.data, ptr_new()]}
               endif else begin
                  if count2 gt 0 then proseries={time:[utime, proseries.time], data:[ptr_new(), proseries.data] }               
               endelse
            endelse
         endelse
      endelse
      utime= utime + retrieval.read_step_minutes*60UL
   endwhile
   return,proseries

end

;************************************************************************************************************
; FUNCTION:    get_netxt_valid
; 
; DESCRIPTION: Searches begining at index <ix> within the timeseries <proseries> in ascending or descending 
;              order for a valid profile.
;              
;              Returns -1 if no valid profile was found within the given intervall.
;              
; PARAMETERS:
;  proseries:  uneffected. Contains timeseries with profiles read from DWD-database
;  
;  ix:         uneffected. Gives a start index where the search have to start from
;  
;  reverse:    uneffected, optional. Gives a direction for the search
;              0: ascending search
;              1: descending search
;              If this parameter is not given, it is set to default value of 0
;  
;  intervall:  uneffected, optional. Gives a range of time in which the search have to be performed.
;              If this parameter is not given, it is set to default value of 3600UL*6UL seconds
; 
; RESULT OF FUNCTION: Returnes an index of a valid profile which was found during the search or
;              -1 if no valid profile was found within the given intervall.  
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function get_netxt_valid, proseries, ix, reverse=reverse,  intervall=intervall
   if N_ELEMENTS(intervall) eq 0 then intervall=3600UL*6UL 
   if N_ELEMENTS(REVERSE) eq 0 then reverse=0 else reverse=1
   if REVERSE eq 1 then begin
      ind=max(ix)
      inc=-1
   endif else begin
      ind=ix[0]
      inc=1
   endelse   
   startind=ind
   res=-1
   while res eq -1 $
       && ind lt N_ELEMENTS(proseries.time) $
       && ind gt 0 $
       && abs_diff(proseries.time[ind], proseries.time[startind]) lt intervall do begin
      if (PTR_VALID(proseries.data[ind]))[0] eq 1 then begin
         res=ind 
      endif else begin
         ind=ind+inc
      endelse
   endwhile
   return, res
end

;************************************************************************************************************
; FUNCTION:    interpol_profile
; 
; DESCRIPTION: Interpolates between two profiles of timeseries and returnes the calculated profile.
;              If just one profile of the timeseries is given, this function is used to generate 
;              and fill a structure with profile (See RESULT OF FUNCTION)
;              
;              Function returnes -1 if one of the given profiles contains an invalid data.
;              
; USES (description of importent routines):
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  hei:        uneffected. Gives a range (array) of heights for the requested profiles
;  
;  proseries:  uneffected. Contains timeseries with profiles read from DWD-database
;  
;  indices:    uneffected. Gives an array of one or two elements which are used as indices for
;              retrieving the profiles (subscripting the pointer array <proseries.data>)
;  
;  proname:    uneffected, optional.Gives a name of the timeseries wich are to be processed
;              If this parameter is not given, it is set to default value of 'MWP'
;              
;  factors:    effected, optional. Gives an array of two elements which are used as
;              factors for each source profile while calculating the interpolated profile.
;              If this parameter is not given while the interpolation is choosed is set to
;              [0.5, 0.5] 
;              
;  agl2asl:    uneffected, optional. Gives the height above the sea level of the location site;
;              If this parameter is not given it is set to default value of 0.0
; 
; RESULT OF FUNCTION: In case of success and depending on timeseries (mwp or raso) returns different  
;              structured profiles (n is number of elements contained in  parameter array <hei>. 
;              Each element of the structure arrays corresponds to the element  given by <hei>):
;              mwp_stru_res={
;                 temp:fltarr(n) ; temperature profiles
;                 rHum:fltarr(n) ; profiles with rel. humidity
;                 lwc :fltarr(n) ; profiles with liquid water content
;              }
;              
;              raso_stru_res={
;                 temp:fltarr(n) ; temperature profiles
;                 rHum:fltarr(n) ; profiles with rel. humidity
;              }     
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function interpol_profile, hei, proseries, indices, proname=proname, factors=factors, agl2asl=agl2asl 
   if N_ELEMENTS(proname) ne 0 then pname=STRUPCASE(proname) else pname='MWP'
   if N_ELEMENTS(agl2asl) eq 0 then agl2asl=0.0
   if (size(proseries))[0] eq 0 && proseries eq -1 then begin
      return, -1
   endif
   
   if N_ELEMENTS(indices) eq 2 then begin
      
      if N_ELEMENTS(factors) ne 2 then factors=[0.5,0.5]
      
      if (PTR_VALID(proseries.data[indices[0]]))[0] eq 0 $
      || (PTR_VALID(proseries.data[indices[1]]))[0] eq 0 then begin
         return, -1
      endif
      if pname eq 'MWP' then begin
         value={$
            temp:factors[0]*interpol((*proseries.data[indices[0]]).temp ,(*proseries.data[indices[0]]).hei, hei+agl2asl)+ $
                 factors[1]*interpol((*proseries.data[indices[1]]).temp ,(*proseries.data[indices[1]]).hei, hei+agl2asl), $ 
            rHum:factors[0]*interpol((*proseries.data[indices[0]]).rHum ,(*proseries.data[indices[0]]).hei, hei+agl2asl)+ $
                 factors[1]*interpol((*proseries.data[indices[1]]).rHum ,(*proseries.data[indices[1]]).hei, hei+agl2asl), $
            lwc :factors[0]*interpol((*proseries.data[indices[0]]).lwc  ,(*proseries.data[indices[0]]).hei, hei+agl2asl)+ $
                 factors[1]*interpol((*proseries.data[indices[1]]).lwc  ,(*proseries.data[indices[1]]).hei, hei+agl2asl)  $
         }   
      endif else begin
         if pname eq 'RASO' then begin
            value={$
               temp:factors[0]*interpol((*proseries.data[indices[0]]).temp ,(*proseries.data[indices[0]]).hei, hei+agl2asl)+ $
                    factors[1]*interpol((*proseries.data[indices[1]]).temp ,(*proseries.data[indices[1]]).hei, hei+agl2asl), $ 
               rHum:factors[0]*interpol((*proseries.data[indices[0]]).rHum ,(*proseries.data[indices[0]]).hei, hei+agl2asl)+ $
                    factors[1]*interpol((*proseries.data[indices[1]]).rHum ,(*proseries.data[indices[1]]).hei, hei+agl2asl)  $
            } 
         endif else begin
            value={$
               temp: factors[0]*interpol((*proseries.data[indices[0]]).temp ,(*proseries.data[indices[0]]).hei, hei+agl2asl)+ $
                     factors[1]*interpol((*proseries.data[indices[1]]).temp ,(*proseries.data[indices[1]]).hei, hei+agl2asl)  $
            }
         endelse
      endelse
   endif else begin
      if N_ELEMENTS(indices) eq 1 then begin
         if (PTR_VALID(proseries.data[indices[0]]))[0] eq 0 then begin
            return, -1
         endif
         if pname eq 'MWP' then begin
            value={$
               temp:interpol((*proseries.data[indices[0]]).temp ,(*proseries.data[indices[0]]).hei, hei+agl2asl), $
               rHum:interpol((*proseries.data[indices[0]]).rHum ,(*proseries.data[indices[0]]).hei, hei+agl2asl), $
               lwc :interpol((*proseries.data[indices[0]]).lwc  ,(*proseries.data[indices[0]]).hei, hei+agl2asl)  $
            }   
         endif else begin
            if pname eq 'RASO' then begin
                           value={$
               temp:interpol((*proseries.data[indices[0]]).temp ,(*proseries.data[indices[0]]).hei, hei+agl2asl), $
               rHum:interpol((*proseries.data[indices[0]]).rHum ,(*proseries.data[indices[0]]).hei, hei+agl2asl)  $
            }
            endif else begin
               value={$
                  temp:interpol((*proseries.data[indices[0]]).temp ,(*proseries.data[indices[0]]).hei, hei+agl2asl)  $         
               }
            endelse
         endelse
      endif else return, -1
   endelse
   ;##############################################
   ;TODO: Hier werden extrapolierten Daten weggeschmissen. Sinnvoll?
   if N_ELEMENTS(value) ne 0 then begin      
      l_limit=min((*proseries.data[indices[0]]).hei)
      h_limit=max((*proseries.data[indices[0]]).hei)
      tmphei =where((hei+agl2asl) gt h_limit, badcount)
      if badcount gt 0 then badhei=tmphei
      tmphei =where((hei+agl2asl) lt l_limit, badcount)
      if badcount gt 0 then begin
         if N_ELEMENTS(badhei) ne 0 then badhei=[badhei,tmphei] else badhei=tmphei 
      endif
      if N_ELEMENTS(badhei) ne 0 then begin
         value.temp[badhei] = !VALUES.F_NAN
         value.rHum[badhei] = !VALUES.F_NAN
         if pname eq 'MWP' then value.lwc[badhei] = !VALUES.F_NAN         
      endif
      return, value 
   endif else return, -1   
end

;************************************************************************************************************
; FUNCTION:    read_profile
; 
; DESCRIPTION: Returns profiles from the timeseries <proseries> at requested time stamp <utime> within a 
;              requested range of height <hei>. Iplemeted for processing timeseries from mwp and raso devices
;              
;              Implements an algorithm for updating the timeseries with profiles from the database. 
;              Searches within the given timeseries for a profile at the requested time stamp. If the 
;              requested time stamp is situated between two profile of the timeseries the function returnes 
;              an interpolated profile.
;              
;              Function returnes -1 if no data could be read from the DWD-database or the time gap 
;              <retrieval.max_interpol_hr> between two profiles is to big to result in a valid interpolation  
;              
; USES (description of importent routines):
;  extend_profile()     updates the timeseries <proseries>
;  clearseries()        deletes the whole timesereis <proseries> or just within a certain time range
;  interpol_profile()   calculates a interpolated profile
;  get_netxt_valid()    finds a valid neighbor profile
;  
; CONDITIONS:  see description of get_dwd_profiles()  
;              
; PARAMETERS:
;  utime:      uneffected. Gives a utc unix time stamp (seconds since the begin of year 1970) 
;              at which profiles should be read.
; 
;  hei:        uneffected. Gives a range (array) of heights for the requested profiles
;  
;  proseries:  effected. Contains timeseries with profiles read from DWD-database
;  
;  retrieval:  uneffected, optional; contains a structure with information about the timeseries 
;              wich are to be processed
;              stru={
;                 proname: string; Name of Profile. Options are: 'MWP' 'RASO' or 'MODEL'
;                 max_interpol_hr: unsigned long; max. number of hours between two data sets 
;                                  in which the interpolation may return acceptable values
;                                  i.e.: (mwp) max_interpol_hr=6, (raso) max_interpol_hr=12
;                                        (model) max_interpol_hr=0
;                 read_step_minutes: unsigned long; defines an intervall in minutes between 
;                                  data sets for a data base query. It is usefull to know how often the
;                                  data is stored into the data base. i.e. timeseries from the mwp device are 
;                                  stored every 10 minutes. 
;              }
;              If this parameter is not given, it is set to default values:
;              proame='MWP', max_interpol_hr=6, read_step_minutes=10
;              
;  agl2asl:    uneffected, optional. Gives the height above the sea level of the location site;  
; 
; RESULT OF FUNCTION: see description of function interpol_profile() in case of success 
;              
;              Returnes -1 if no data could be read from the DWD-database or the time gap <retrieval.max_interpol_hr> 
;              between two profiles is to big to result in a valid interpolation
; 
; NOTE (ToDo): If empty time seires <proseries> are given or will be deleted in case of "too old" data, the 
;              algorithm tries to read new profiles from the DWD-database for the next 24 hours beginning
;              from <utime>. This behavior can cause that the time series <proseries> might hold an 
;              "invalid" data (-1)  from the future. To prevent the algorithm to do so, the call of this function
;              should follow the conditions descripted in function get_dwd_profiles(). Alternatively the 
;              algorithm can be fine tuned by changing the variable <UPDATE_INTERVALL> wich is set to default
;              value of 24 hours. Keep in mind that i.e. data from raso-device is available only every 6 hours.      
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function read_profile, utime, hei, proseries, retrieval=retrieval, agl2asl=agl2asl
   ONEHOUR=3600UL
   MAXTIMEGAP=ONEHOUR*24UL*7UL
   UPDATE_INTERVALL=24
   if N_ELEMENTS(retrieval) eq 0 then begin
      retrieval.proname='MPW'  & retrieval.max_interpol_hr=6UL  & retrieval.read_step_minutes=10UL      
   endif
   INTERPOLGAP=retrieval.max_interpol_hr*ONEHOUR
   
   if N_ELEMENTS(agl2asl) eq 0 then agl2asl=0.0 
   
   ; if profile empty, then load profiles from utime + UPDATE_INTERVALL hours 
   if (size(proseries))[0] eq 0 && proseries eq -1 then proseries=extend_profile(proseries, retrieval, /direction, nHours=UPDATE_INTERVALL, starttime=utime)
   if (size(proseries))[0] eq 0 && proseries eq -1 then return, -1
   
   ; delete too "old" time series for saving the memory space
   if (min(proseries.time) lt utime) && (abs_diff(utime,min(proseries.time)) gt MAXTIMEGAP) then begin
      proseries=clearseries(proseries, FROMTO=[min(proseries.time), utime-ulong(MAXTIMEGAP/2)])
      if (size(proseries))[0] eq 0 && proseries eq -1 then proseries=extend_profile(proseries, retrieval, /direction, nHours=UPDATE_INTERVALL, starttime=utime)
      if (size(proseries))[0] eq 0 && proseries eq -1 then return, -1
   endif
   
   
   ix1=where(proseries.time eq utime, count1)
   if count1 ne 0 then begin
      ; is there a valid profile at the requested time stamp?
      ix1=ix1[0]
      if (PTR_VALID(proseries.data[ix1]))[0] ne 0 then begin
         return,  interpol_profile(hei, proseries, ix1, proname=retrieval.proname, agl2asl=agl2asl)
      endif else ix2=ix1   
   endif else begin
      ; find neighbor profiles and update(extend) the time series <proseries> if there are no neighbors    
      found=2
      while found gt 0 do begin
         ix1=where(proseries.time le utime, count1)
         ix2=where(proseries.time gt utime, count2)
         if count1 eq 0 then begin
            ; Extend proseries to the left
            if (abs_diff(utime,min(proseries.time))) gt MAXTIMEGAP then begin
               ; extend to the left and delete too "young" data 
               nHours=UPDATE_INTERVALL+max([1, ulong(retrieval.read_step_minutes*60UL/ONEHOUR)])
               proseries=extend_profile(clearseries(proseries), retrieval, /direction, nHours=nHours, starttime=utime-retrieval.read_step_minutes*60UL)
            endif else begin
               nHours=max([ulong(abs_diff(utime,min(proseries.time))/ONEHOUR)+1, ulong(retrieval.read_step_minutes*60UL/ONEHOUR)])
               proseries=extend_profile(proseries, retrieval, direction=-1, nHours=nHours)
            endelse            
         endif else begin
            if count2 eq 0 then begin
               ; Extend proseries to the right
               if (abs_diff(utime,max(proseries.time))) gt MAXTIMEGAP then begin
                  ; extend to the right and delete too "old" data
                  nHours=UPDATE_INTERVALL+max([1, ulong(retrieval.read_step_minutes*60UL/ONEHOUR)])
                  proseries=extend_profile(clearseries(proseries), retrieval, /direction, nHours=nHours, starttime=utime-retrieval.read_step_minutes*60UL)
               endif else begin
                  nHours=max([ulong(abs_diff(utime,max(proseries.time))/ONEHOUR)+1,ulong(retrieval.read_step_minutes*60UL/ONEHOUR)])
                  proseries=extend_profile(proseries, retrieval, /direction, nHours=nHours)
               endelse 
            endif else found =0 
         endelse   
         found = found - 1
      endwhile      
   endelse  
   
   ; find really valid neighbors
   ix1=get_netxt_valid(proseries, ix1, /reverse)
   ix2=get_netxt_valid(proseries, ix2)
   
   if ix1 ge 0 && ix2 ge 0 && abs_diff(proseries.time[ix1],proseries.time[ix2]) lt INTERPOLGAP then begin
      ; two valid neighbors found
      w1=double(proseries.time[ix2]-utime)/double(proseries.time[ix2]-proseries.time[ix1])
      w2=double(utime-proseries.time[ix1])/double(proseries.time[ix2]-proseries.time[ix1])
      return,  interpol_profile(hei, proseries, [ix1, ix2], factors=[w1,w2], proname=retrieval.proname, agl2asl=agl2asl)  
   endif else begin
      if ix1 ge 0 && ix2 ge 0 then begin
         ; timegap between neighbors is too big. Take the closest neighbor or leave if the closest is still far.  
         if abs_diff(proseries.time[ix1],utime) gt abs_diff(proseries.time[ix2],utime) then begin
            if abs_diff(proseries.time[ix2],utime) lt INTERPOLGAP then begin
               return,  interpol_profile(hei, proseries, ix2, proname=retrieval.proname, agl2asl=agl2asl)
            endif
         endif else begin
            if abs_diff(proseries.time[ix1],utime) lt INTERPOLGAP then begin
               return,  interpol_profile(hei, proseries, ix1, proname=retrieval.proname, agl2asl=agl2asl)
            endif
         endelse            
      endif else begin
         if ix1 lt 0 && ix2 lt 0 then begin
            ; no neighbor found
            return, -1
         endif else begin
            ; just one neighbor found
            if ix1 ge 0 && abs_diff(proseries.time[ix1],utime) lt INTERPOLGAP then begin
               return,  interpol_profile(hei, proseries, ix1, proname=retrieval.proname, agl2asl=agl2asl)
            endif
            if ix2 ge 0 && abs_diff(proseries.time[ix2],utime) lt INTERPOLGAP then begin
               return,  interpol_profile(hei, proseries, ix2, proname=retrieval.proname, agl2asl=agl2asl)
            endif
         endelse
      endelse
   endelse   
   print, 'read_profile: no data found at requested time' & return, -1

end

;************************************************************************************************************
; FUNCTION:    get_dwd_profiles
; 
; DESCRIPTION: Returns temperature (temp), rel. humidity (rHum) and liquid water content (lwc) profiles at 
;              requested time stamp, given by the parameter <utime> within requested range of height, 
;              given by  the paramter <hei>.
;              
;              Three kinds of retrievals are available for reading the profiles:
;              a) microwave profiler (mwp): contains temp, rHum and lwc profiles
;              b) radio sonde (raso): contains temp and rHum profiles
;              c) model: contains only a modeled temp profile
;              
;              The function reads profiles succesive from a prefered retrieval source. The profiles are 
;              stored as timeseries in global common viriables wich are updated by an algorithm while 
;              the function is accessing them. 
;              
;              The updateing algorithm reads the profiles from the DWD-database with an adjustable 
;              intervall (<read_step_minutes>) between data sets and stores them into the global common 
;              variables. It deletes also "too old" values from these common variables.
;              
;              The function returns profiles at any time stamp (except those mentioned in CONDITIONS a))
;              If the requested time stamp is situated between two data sets, the function will return
;              an interpolated profile.
;                
;              In case the timeseries stored in a common variable contain no data for the requested time 
;              stamp (and there is really no data in the DWD-database), or the time gap (<max_interpol_hr>)
;              between two valid data sets is too big, a next source will be selected from the 
;              list of prefered sources. The list of prefered sources is read from an ini-file or is 
;              given by an optional parameter <prefered_retrieval>
;              
;              If all sources should contain no data, the function returns a raughly estimated temperature
;              profle. The estimation depends on the optional parameters <MeltHei> and <outdoortemp2metlhei>
;              
; CONDITIONS:  a) This function can only be run on on the PC of DWD cloudradar or DWD auswertung.
;              b) This function can only return not estimated profiles at time stamps which are less then 
;                 the actual time at least 24 hours. This is the time delay the profiles apear in the data base.
;              If the mentioned conditions are not met, the functin will return a raughly estimated 
;              temperetaure profile
;              
; USES (description of importent routines):       
;  read_inifile()    returnes a list of prefered reftrieval sources
;  read_profile()    returnes a prefered or an alternative profile
;  read_model()      returnes a modeled temperature profile
;              
; PARAMETERS:
;  hei:        uneffected. Gives a range (array) of heights for the requested profiles
;  
;  utime:      uneffected. Gives a utc unix time stamp (seconds since the begin of year 1970) 
;              at which profiles should be read.
;              
;  MeltHei:    uneffected, optional. Gives a height of the melting layer. If this parameter is not given
;              it is set to 1800 meters.
;              
;  outdoortemp2metlhei: uneffected, optional. Gives a scalling factor for the estimation of temperature 
;              profiles. Set to default 190
;              
;  agl2asl:    uneffected, optional. Gives the height above the sea level of the location site
;  
;  done:       effected, optional. Returns an indication about what profile is returned as the result of function.
;                 done= 1  => a mwp profile is returned
;                 done= 2  => a raso profile is returned
;                 done= 3  => a modeled profile is returned
;                 done=-1  => an estimated temperature profile is calculated using
;                             MeltHei=1800, outdoortemp2metlhei=outdoortemp2metlhei
;                 done=-11 => an estimated temperature profile is calculated using
;                             MeltHei=1800, outdoortemp2metlhei=190
;                 done=-2  => an estimated temperature profile is calculated using
;                             MeltHei=MeltHei, outdoortemp2metlhei=outdoortemp2metlhei
;                 done=-21 => an estimated temperature profile is calculated using
;                             MeltHei=MeltHei, outdoortemp2metlhei=190        
;                             
;  prefered_retrieval: uneffected, optional. Gives a list of prefered sources of profile retrieval.
;              This parameter expects an array of structures with 3 elemets. The index number
;              of these elements gives a fall back ordering which is used by the algorithm if the
;              first kind of retirieval from this list contains no data. Define following structures to fill
;              this array
;              stru={
;                 proname: string; Name of Profile. Options are: 'MWP' 'RASO' or 'MODEL'
;                 max_interpol_hr: unsigned long; max. number of hours between two data sets 
;                                  in which the interpolation may return acceptable values
;                                  i.e.: (mwp) max_interpol_hr=6, (raso) max_interpol_hr=12
;                                        (model) max_interpol_hr=0
;                 read_step_minutes: unsigned long; defines an intervall in minutes between 
;                                  data sets for a data base query. It is usefull to know how often the
;                                  data is stored into the data base. i.e. timeseries from the mwp device are 
;                                  stored every 10 minutes. 
;              }         
; 
; RESULT OF FUNCTION:
;              Depending on the prefered kind of retrieval or in case the algorithm has fallen
;              back to another kind or retrieval the function returns different structured 
;              profiles. Which profile is returned is indicated by the optional parameter <done>
;              
;              The function can return following structures (n is number of elements contained in
;              parameter array <hei>. Each element of the structure arrays corresponds to the element
;              given by <hei>):
;              
;              mwp_stru={        ; done=1
;                 temp:fltarr(n) ; temperature profiles
;                 rHum:fltarr(n) ; profiles with rel. humidity
;                 lwc :fltarr(n) ; profiles with liquid water content
;              }
;              
;              raso_stru={       ; done=2
;                 temp:fltarr(n) ; temperature profiles
;                 rHum:fltarr(n) ; profiles with rel. humidity
;              }
;              
;              model- and estimated_stru
;                 temp:fltarr(n) ; temperature profiles
;              }
;  
; AUTHOR:      Alexander Partus (partus@metek.de)
; DATE:        1.04.2010
;************************************************************************************************************
function get_dwd_profiles, hei, utime, MeltHei=MeltHei, done=done, $
   agl2asl=agl2asl,outdoortemp2metlhei=outdoortemp2metlhei, prefered_retrieval=prefered_retrieval

   common dwd_profiles, mwp_profile, raso_profile, pref_retr
   common magic4chs,magic4chs,FzkPdsFmt,syspar
   if N_ELEMENTS(mwp_profile) eq 0 then mwp_profile=-1
   if N_ELEMENTS(raso_profile) eq 0 then raso_profile=-1
   
   if N_ELEMENTS(prefered_retrieval) ne 0 then pref_retr=prefered_retrieval
   ;if (N_ELEMENTS(pref_retr) eq 0) || ((N_ELEMENTS(pref_retr) eq 1) && (pref_retr eq -1)) then begin
      pref_retr=read_inifile()
   ;end        
     

   if n_elements(agl2asl) ne 1 then agl2asl=0.0
      
   
   nowis = ulong(systime(/sec,/utc))
   ONEHOUR=3600UL
   DWD_DELAY = ONEHOUR*24UL    
    
   host=getenv('HOSTNAME')
   if host eq 'neander' $
   || strpos(strupcase(host),'PLGFE3F003') ge 0 $
   || strpos(strupcase(host),'DWD') ge 0 $
   || host eq 'radarserver' $
   || ( host eq '' and strmid(syspar.iam,0,4) eq 'DWD1') then begin
   
      if (abs_diff(nowis,utime) lt DWD_DELAY) || (utime gt nowis) then begin
        print, "there are no coresponding profiles yet. Try later"
        res=-1
      endif else begin
        mode_ix=0 
        while mode_ix lt N_ELEMENTS(pref_retr) do begin
        print, 'reading '+pref_retr[mode_ix].proname+'-profile at '+unixtime2bdate(utime, nice=2, /UTC)
           case pref_retr[mode_ix].proname of
             'MWP'  : begin
                        res=read_profile(utime, hei, mwp_profile, retrieval=pref_retr[mode_ix],  agl2asl=agl2asl)
                        done=1
                      end              
             'RASO' : begin
                        res=read_profile(utime, hei, raso_profile,retrieval=pref_retr[mode_ix],  agl2asl=agl2asl)
                        done=2
                      end  
             'MODEL': begin
                        res=read_model(utime, hei, agl2asl=agl2asl)
                        done=3
                      end  
             else  :begin
                      print, 'Error: trying to load unknown profile.'
                      res=-1
                    end  
           endcase
           if (N_ELEMENTS(res) eq 0) || ((size(res))[2] ne 8) then mode_ix++ else break
        endwhile 
      endelse
   endif
   
   if ((size(res))[2] ne 8) then begin
     if n_elements(MeltHei) ne 1 || MeltHei lt -80000 then begin
        xMeltHei=1800.
        done=-1
     endif else begin
        xMeltHei=MeltHei
        done=-2
     endelse
     if n_elements(outdoortemp2metlhei) ne 1 then begin
         outdoortemp2metlhei=190.
         done = (done * 10) -1
     endif
     res = {temp:(xMeltHei-hei) * (1. / outdoortemp2metlhei)}
           
   endif
    

   return,res

end
