
pro read_ceilo_dat_chm15k, filename, utime, bases

; procedure reads Jenoptik CHM15k Ceilometer dat-file(s), which contains cloud bases and other information, and
; returns arrays utime (time series) and bases (cloud base height) as called-by-reference parameters
;
; Example of file format (first 3 lines)
;
;X1TA;8;015;19.01.09;00:00;00352;NODET;NODET;0131;NODT;NODT;00391;00483;+000;m ;01;00000000;16;CHM060012;00023;NODET;NODET;0057;NODT;NODT;99999;0156;0880;OK;00000000;2756;2883;2976; 105;0001;013540;0210;0078703;0221;0232;NODET;NODET;06;00131;NODET;NODET;
;X1TA;8;015;19.01.09;00:00;00342;NODET;NODET;0141;NODT;NODT;00384;00483;+000;m ;01;00000000;16;CHM060012;00018;NODET;NODET;0021;NODT;NODT;00029;0156;0880;OK;00000000;2756;2883;2976; 105;0001;013540;0210;0080654;0221;0232;NODET;NODET;22;00141;NODET;NODET;
;X1TA;8;015;19.01.09;00:00;00348;NODET;NODET;0147;NODT;NODT;00392;00495;+000;m ;01;00000000;16;CHM060012;00012;NODET;NODET;0018;NODT;NODT;99999;0156;0880;OK;00000000;2756;2883;2977; 105;0001;013540;0210;0079873;0221;0232;NODET;NODET;37;00147;NODET;NODET;

; Syntax:
;
; result = read_ceilo_dat_CHM15k(filename)
;
; Arguments:
;
; filename  : full path to the file, which contains the ceilometer data
;
; Example:
;
; result = read_ceilo_dat_chm15k('/home/alex/work/dwd/ceilo/CHM060012_2010-04-14.dat')
;
; UG 08.03.2005

line=''

openr, lun, filename, /get_lun, err = error

if error eq 0 then begin

  ii = 0
  repeat begin
    ii = ii+1
    readf, lun, line
  endrep until (eof(lun))
  close, lun
  np = ii

  ;define arrays
  utime = lonarr(np)
  bases = fltarr(3,np)
  bases[0,*] = replicate(!values.f_nan, np) ; firts cloud base height in m
  bases[1,*] = replicate(!values.f_nan, np) ; second cloud base height in m
  bases[2,*] = replicate(!values.f_nan, np) ; third cloud base height in m

  openr, lun, filename, /get_lun

  for ii=0, np-1 do begin

    readf, lun, line ; reads data line

    var   = strsplit(line, ';',/extract)

    sdate  = var[3]
    day   = strmid(sdate,0,2)
    month = strmid(sdate,3,2)
    year  = '20'+strmid(sdate,6,2)

    stime = var[4]
    hour  = strmid(stime,0,2)
    min   = strmid(stime,3,2)
    sec   = var[42]

    utime[ii]= (JULDAY(month, day, year)-JULDAY(01,01,1970))*24UL*60UL*60UL+hour*3600UL+min*60UL+sec 

    if n_elements(d15) ne 1 then d15 = ulong( utime[ii] gt 1292371200 )
    if (var[5+d15] ne 'NODET') && (var[5+d15] ne '-----') && var[8+d15] lt 9998 then begin
        bases[0,ii] = var[5+d15]
    endif
    if (var[6+d15] ne 'NODET') && (var[6+d15] ne '-----') && var[9+d15] lt 9998 then begin
        bases[1,ii] = var[6+d15]
    endif
    if (var[7+d15] ne 'NODET') && (var[7+d15] ne '-----') && var[10+d15] lt 9998 then begin 
        bases[2,ii] = var[7+d15]
    endif
  endfor

  close, lun
endif else print, 'Could not find Ceilometer DAT-file (check path settings!)'

end



