pro viewpds,arguments
; Tool for visualizing profiles of moments and Spectra of the MIRA-36 cloud radar
; provided as pdf files which are saved by the radarserver. 
;
; viewpds can be called from the IDL prompt or from a unix shell:
; 
; unix> idl viewpds infile=data/20040602_1000.pds:data/20040602_1200.pds \
;                extract=p1:p2:sp1 bdate_begin=DT120000 bdate_end=180000
;
; IDL > viewpds, 'infile=data/20040602_1000.pds extract=p1:p2:sp1 bdate_begin=110000'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; extract     An array of tag names which specify which profiles should be extracted.
;             If omitted all profiles which are in the pds file are extracted.
;             If the file is biger than the system memory it is useful to extract
;             only a subset of the profiles. The following 'tags' are allowed: 
;             p1, p2, v1, v2, r1, r2, sp1, sp2.
;
; bdate_begin 
; bdate_end   Specify a date-time range from which data should be extracted.
;             Both must be provided as String in the form yymmddHHMMSS. If leading
;             chars are omitted they are replaced to match the date-time of the
;             first dwell contained in the pds-file.
;
; max_dwells  limits the number of dwells which is loaded per file.
; max_mem     viewpds will stopp goging data if this much memory is used.
;
; After launching the pds files are gorged and a cmd line interface is launched.
; Different visualizations can be created by the following commands:
;
;-----------------------------------------------------------------------------------------------------------------
;     View PDS > sp [sp1|sp2]
;
; Plot the profile of spectra at dwell number itime as image plot. The profile of velocities from the moment data 
; is oplotted into the image plot. The dwellnumber itime can be selected by one of the following commands:
;
;     View PDS > + [increment]
;     View PDS > - [decrement]
;     View PDS > ++                # go to the begining of the next file if multible files where loaded
;
; Note multible commands seperated by " ; " (blancs are necessary) can be given in one line:
;
;     View PDS > + ; sp  return return return ...
;
; Like this profiles can be stepped. This works because blanc command lines cause repeating 
; of the preveous (non-blanc) command line.
;
; If multible files where loaded then ++ (--) seeks to the begining of the next 
; (preveous) file.
;
;-----------------------------------------------------------------------------------------------------------------
;     View PDS > trc [p1|p2|v1|v2|s1|s2|ldr]
;
; Plot a Time Range Cross section of the profiles idicated by the first argument.
; The default value of the z-range of each type of profiles is stored in the
; parameter <f>range where <f> is the first letter of the profiles name. For 
; example the default value of the z-range for the ldr plots cam be changed by
;
;    View PDS > set lrange -30 -1 ; trc ldr
;
;-----------------------------------------------------------------------------------------------------------------
;    View PDS > set <parameter> <valu1> [<value2> ...]
;
; Change the default value of a plotting parameter. If <value1> is omited the
; current value is plotted. All the plotting parameters which 
; are available can be shown by
;
;    View PDS > set help
;-----------------------------------------------------------------------------------------------------------------
;    View PDS > tr <begin_hour> <end_hour>
;
; limits the time range of the trc plots. <begin_hour> <end_hour> are given as decimal numbers:
; 17.5 means 17:30. The hours ar countet from 00:00 of the first day contained in the data
; which is loaded.
;-----------------------------------------------------------------------------------------------------------------
;    View PDS > postscript <filename>
;
; Directs the output of the following plot commands to the given postscript filename.
; for flushing and closing the postscript file the command
;
;    View PDS > close
;
; must be given. After that the plot out put is directed to the screen again.
;----------------------------------------------------------------------------------------------------------------
;    View PDS > png <filename>
;
; exports the screen to png-format
;-----------------------------------------------------------------------------------------------------------------
;    View PDS > info
;
; gives some information from the file header
;-----------------------------------------------------------------------------------------------------------------
;    View PDS > x
;
; exits ViewPDS
;----------------------------------------------------------------------------------------------------------------
; If no command is recognized then the command is executed by the IDL interpreter.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

startup_device=!d.name

; Arguments can be provided by two methods:
; - If viewpds is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)
kp = { $
    infile:'',$
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    extract:'default', $
    bdate_begin:'default', $
    bdate_end:'default', $
    progress_bar:0,$
    calibr:[-2000.,-2000.],$
    max_mem:400000000L, $
    max_headers:500L, $
    max_dwells:70000L $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam) ; this is only for setting the common bloc variables
common magic4chs,magic4chs,FzkPdsFmt,syspar

infiles = strsplit(kp.infile,':',/extract)

if n_elements(infiles) eq 1 and infiles[0] eq '' then begin
    infiles = DIALOG_PICKFILE(/read,/must_exist,/multiple_files, path = path, filter='*.pds')
endif

extracts = strsplit(kp.extract,':',/extract)

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
hr:[230.,13000.],$          ; see comments below!
itr:[0,100L], $
badcol :15, $
srange:[-15,75.], $
prange:[-30,70.], $
zrange:[-55,35.], $
vrange:[!values.f_nan,!values.f_nan], $
rrange:[0.0,4.], $
lrange:[-35.,0.], $
raute:1,$
min_txn_pow:10., $
needrms:0,$
min_npw:.3,$
rec_err_good_bits:'0000'X,$
rec_err_bad_bits:'0000'X, $
xtu:'Minutes',$
tolerable_gap:11d0, $
takeat_gap:'right', $
nbin:100,$
zrsym:3,$
charsize:1.1, $
thick:1.0, $
b4tit:'',$
ti:'!3',$
colt:'smooth' $
}
argv2stru,argv,pp

pph={ $
hr:      "Height range for plotting", $
itr:     "Range of dwell numbers which are shown in trc-plots", $
badcol : "Grey, color for plotting values which are not plausible.", $
srange:  "Zrange for spectra", $
prange:  "Zrange for snr profiles", $
zrange:  "Zrange for dbz", $
vrange:  "Zrange for velocities profiles", $
rrange:  "Zrange for    Peak with (RMS) profiles", $
lrange:  "Zrange for    LDR profiles", $
raute:  "If set the 2nd moment calculated by the DSP is added to the spectra plots.", $
min_txn_pow: "Only profiles are plotted by trc in which srvinfo.tpow (transmitted avg power * 100) is grater than min_txn_pow", $
min_npw: "Only profiles are plotted by trc in which srvinfo.npw1 > min_npw", $
needrms: 'If set values are only plotted when RMS is defined (!= 10000)',$
rec_err_good_bits: "Only profiles are plotted by trc in which rec_err_good_bits in tsrvinfo.re_err are set", $
rec_err_bad_bits: "Only profiles are plotted by trc in which rec_err_bad_bits in tsrvinfo.re_err are not set", $
xtu:     "Parameter to control the number of ticks on the time axis. 'minutes'|'time'|'hours'..", $
tolerable_gap: "A gap in the time sequence will be detected when difference between the time stamps of two dwells is greater than tolerable_gap*median dwelltime", $
takeat_gap: "Describes how itr should be changed when a gap is detected", $
nbin: 'Number of bins for histograms',$
zrsym: 'Symbols for isolated points in zr-plots',$
charsize: "See graphic keywords in IDL documentation", $
thick:   "See graphic keywords in IDL documentation", $
b4tit:  "Can be used to pass a Hershly-style font directive used only for title lines", $
ti:     "Can be uses to pass a Hershly-style font directive used for all annotations", $
colt:   "Color table, the predefined colortables reside in the directory idl/colors" $
}

startcolor=30
ocolt='none'
td=1000.

!p.multi=[0,1,1]
if !d.name eq "X" then begin
    if !d.window ge 0 then wdelete
    screen_size=get_screen_size()
    scale=screen_size/[1280.,1024]
    maxcolor=512
    window,/free,retain=2,colors=maxcolor,title="viewpds",xpos=90*scale[0],ypos=200*scale[1], $
    xsize=1180*scale[0],ysize=800*scale[1]
endif

zfiles = n_elements(infiles)
zheaders=kp.max_headers
last_zt = 0L
iheader = 0L
irg=30
ifile=0L
while ifile lt zfiles do begin
   infile=infiles[ifile]
   gorge_pds_file, infile, ppar, hdr, parr_srvinfo, parr_data, extract=extracts, bdate_begin=kp.bdate_begin, $
        bdate_end=kp.bdate_end,max_dwells=kp.max_dwells,progress_bar=kp.progress_bar,max_mem=kp.max_mem,lun=lun,gziped=gziped
   if iheader eq 0 then begin
       hdrs=replicate(hdr,zheaders)
       ppars=replicate(ppar,zheaders)
       ztimes = lonarr(zheaders)
   endif
   this_zt = n_elements(parr_srvinfo) * ptr_valid(parr_srvinfo[0])
   if this_zt gt last_zt then begin
       ztimes[iheader] = n_elements(parr_srvinfo) * ptr_valid(parr_srvinfo[0])
       hdrs[iheader] = hdr
       ppars[iheader] = ppar
       if ztimes[iheader] gt 0 then begin
           print,infile,': ',unixtime2bdate((*parr_srvinfo[last_zt]).TIME_T,nice=2,utc=1), $
                   ' - ',unixtime2bdate((*parr_srvinfo[ztimes[iheader]-1]).TIME_T,nice=2,utc=1)
       endif
       ++iheader
   endif
   if lun eq -1000 then ++ifile
   last_zt = this_zt
endwhile
zheaders=iheader

max_ave = max(hdrs[*].ave,min=min_ave)
if max_ave ne min_ave then print,'Files with different averaging time have been loaded. The time axis of trc plots will be wrong. Use ++, -- !'
max_zmrg = max(hdrs[*].mom_zrg,min=min_zmrg)
if max_zmrg ne min_zmrg then print,'Files with different number of range gates have been loaded. Use ++, -- !'

ztime = n_elements(parr_srvinfo) * ptr_valid(parr_srvinfo[0])
if ztime eq 0 then message,'No Data gorged'
unix_times = lonarr(ztime) 
for itime=0L, ztime-1L do unix_times[itime] = (*parr_srvinfo[itime]).TIME_T
if min(unix_times[1L:ztime-1L]-unix_times[0L:ztime-2L]) lt 0 then print,'time stamps are not in increasing order, pds files may be swapped!'
hours = unix_times/3600d0
first_day_0000 = long(hours[0]/24d0)*24d0
hours = hours - first_day_0000
pp.itr = [0,ztime-1L] ; range of dwell numbers which are shown in trc-plots
juldate = unixtime2jul(unix_times)

txn_power = fltarr(ztime)
for itime=0L, ztime-1L do txn_power[itime] = (*parr_srvinfo[itime]).tpow

txn_err = intarr(ztime)
for itime=0L, ztime-1L do txn_err[itime] = (*parr_srvinfo[itime]).te_err

rec_err = intarr(ztime)
for itime=0L, ztime-1L do rec_err[itime] = (*parr_srvinfo[itime]).rc_err

npw = fltarr(ztime,2)
for itime=0L, ztime-1L do npw[itime,0] = (*parr_srvinfo[itime]).npw1
for itime=0L, ztime-1L do npw[itime,1] = (*parr_srvinfo[itime]).npw2

cpw = fltarr(ztime,2)
for itime=0L, ztime-1L do cpw[itime,0] = (*parr_srvinfo[itime]).cpw1
for itime=0L, ztime-1L do cpw[itime,1] = (*parr_srvinfo[itime]).cpw2

ztime=n_elements(parr_data)
itime=0L
ipcs=0L
last_draw_arg='trc'
while 1 do begin
    if ocolt ne pp.colt then begin
        loadcolor,startcolor,colorfile=pp.colt,zcolor=zcolor 
        ocolt=pp.colt
    endif
    
    arg = myprompt('View PDS >')
artarg:
    cmd = arg[0] &  narg=n_elements(arg)
    case 1 of
        cmd eq 'redraw' or cmd eq 'r': begin
            arg = last_draw_arg
            goto,artarg
        end
        cmd eq 's': begin
            hdr = hdrs[first(where(itime lt ztimes))] ; pick the header of the file from which itime was loaded
            last_draw_arg = arg
            if narg eq 3 && strlen(arg[1]) eq 12 then begin
                btime = arg[1] & arg[1]=arg[2] & arg[2]=btime
            endif
            if narg ge 2 then irg=fix(arg[1]) 
            irg = (irg < (hdr.raw_zrg-1) ) > 0
            
            if narg ge 3 then begin 
                utime=bdate2unixtime(arg[2])
                dutime = min(abs(unix_times-utime),itimex)
                if dutime gt hdr.ave then begin
                    print,'Given btime does not match. Best match: itime=',itimex,' deviation',unix_times[itimex]-utime
                endif
                itime=itimex
            endif
            dsp2vel = hdr.la*hdr.prf
            nffth = hdr.nfft/2
            if syspar.pds_spectra_centered then begin
                toord = (hdr.nfft-1L - lindgen(hdr.nfft)) mod hdr.nfft  ; eventuell das -1L weglassen.
            endif else begin
                toord = (hdr.nfft-1L - [lindgen(nffth)+nffth,lindgen(nffth)]) mod hdr.nfft ; idx-vector for centering the spectra to zero velocity
            endelse
            spcsco = *(*parr_data[itime]).sp1
            spcscx = *(*parr_data[itime]).sp2
            veloaxis = syspar.dwd_me_spc*0.5*(findgen(hdr.nfft)-nffth+1)/float(nffth)*hdr.vuar
            if finite(pp.vrange[0]) then vlra=syspar.dwd_me_spc*pp.vrange else vlra=syspar.dwd_me_spc*[-0.5,.5]*hdr.vuar
            tit = unixtime2bdate(unix_times[itime], nice=0,utc=1) + ' rg:' + string(irg) + ' Range:'+ string(irg*hdr.delta_h+hdr.raw_h_min)
            if finite(spcsco[0]) then begin
                plot,veloaxis,db(spcsco[toord,irg]),yrange=pp.srange,xtit='Velocity [m/s]',ytit='rel. Power [dB]',tit=tit,psym=10,xrange=vlra,xst=1,yst=1,xticklen=1,xgrid=1,yticklen=1,ygrid=1
                ;oplot,syspar.dwd_me*0.5*hdr.vuar*[-1,1],db(npw[itime,0]*[1,1]),linest=2
                if finite(spcscx[0]) then begin
                    oplot,veloaxis,db(spcscx[toord,irg]),color=1,psym=10
                    ;oplot,syspar.dwd_me*0.5*hdr.vuar*[-1,1],dB(npw[itime,1]*[1,1]),linest=2,color=1
                endif
            endif else begin
                plot,veloaxis,db(spcscx[toord,irg]),yrange=pp.srange,xtit='Velocity [m/s]',ytit='rel. Power [dB]',tit=tit,psym=10,xrange=vlra,xst=1,yst=1,xticklen=1,xgrid=1,yticklen=1,ygrid=1
                ;oplot,syspar.dwd_me*0.5*hdr.vuar*[-1,1],db(npw[itime,1]*[1,1]),linest=2
            endelse
        end
        cmd eq 'sp': begin
            last_draw_arg = arg
            if narg ge 2 then tag_sp=arg[1] else tag_sp='sp1'
            ; plot a profile of spectra and oplot the profile of velocities from the moment data 
            hdr = hdrs[first(where(itime lt ztimes))] ; pick the header of the file from which itime was loaded
            dsp2vel = hdr.la*hdr.prf
            nffth = hdr.nfft/2 
            if syspar.pds_spectra_centered then begin
                toord = hdr.nfft-1L - lindgen(hdr.nfft)
            endif else begin
                toord = hdr.nfft-1L - [lindgen(nffth)+nffth,lindgen(nffth)] ; idx-vector for centering the spectra to zero velocity
            endelse
            raw_height = findgen(hdr.raw_zrg) * hdr.delta_h + hdr.raw_h_min
            raw_ihr=[first(where(raw_height ge pp.hr[0])),last(where(raw_height le pp.hr[1]))]
            if min(raw_ihr) lt 0 or raw_ihr[0] ge raw_ihr[1] then begin
                print,'height range pp.hr',pp.hr,' should be between',first(raw_height),last(raw_height)
                break
            endif
            spcs_plt = un_gorge(hdr,parr_data,itime=itime,irg=raw_ihr,tag=tag_sp)
            if not finite(spcs_plt[0,0]) then break
            nfftme = hdr.nfft-1
            if finite(pp.vrange[0]) then begin
                vlra=pp.vrange
                fullvlra=[-0.5,.5]*hdr.vuar
                iaspc=round(nfftme*(vlra[0]-fullvlra[0])/(fullvlra[1]-fullvlra[0]))
                ibspc=round(nfftme*(vlra[1]-fullvlra[0])/(fullvlra[1]-fullvlra[0]))
            endif else begin
                vlra=[-0.5,.5]*hdr.vuar
                iaspc=0 & ibspc=nfftme
            endelse
            mimage, db(spcs_plt[toord[iaspc:ibspc],*]), vlra, raw_height[raw_ihr]/td , pp.srange, $
                zcolor, startcolor,pp.badcol,charsize=pp.charsize,thick=pp.thick, $
                title='Profile of Spectra ' + unixtime2bdate(unix_times[itime], nice=0,utc=1), $
                ztitle='Intensity',xtitle='Velocity [m/s]',$
                heitit='Range [km]'
            if pp.raute then begin
                tag_velo = 'v'+strmid(tag_sp,2,1)
                vr_plt = syspar.dwd_me * un_gorge(hdr,parr_data,itime=itime,tag=tag_velo)
                if not finite(vr_plt[0]) then break
                mom_height = [findgen(hdr.mom_zrg) * hdr.delta_h + hdr.mom_h_min,15000.,15000.]
                ;oplot, vr_plt , mom_height / td + 999.*(abs(vr_plt) gt 100.),color=0, max=900.,$
                ;    thick = pp.thick
                oplot, vr_plt , mom_height / td + 999.*(abs(vr_plt) gt 100.),color=0, max=900.,$
                    thick = pp.thick, psym=4
            endif
        end
        cmd eq 'ss': begin
            last_draw_arg = arg
            if narg ge 2 then tag_sp=arg[1] else tag_sp='sp1'
            ; plot a sequence of spectra and oplot the profile of velocities from the moment data 
            irg = (irg < (hdr.raw_zrg-1) ) > 0
            hdr = hdrs[first(where(itime lt ztimes))] ; pick the header of the file from which itime was loaded
            dsp2vel = hdr.la*hdr.prf
            nffth = hdr.nfft/2 
            if syspar.pds_spectra_centered then begin
                toord = hdr.nfft-1L - lindgen(hdr.nfft)
            endif else begin
                toord = hdr.nfft-1L - [lindgen(nffth)+nffth,lindgen(nffth)] ; idx-vector for centering the spectra to zero velocity
            endelse
            raw_height = findgen(hdr.raw_zrg) * hdr.delta_h + hdr.raw_h_min
            raw_ihr=[first(where(raw_height ge pp.hr[0])),last(where(raw_height le pp.hr[1]))]
            if min(raw_ihr) lt 0 or raw_ihr[0] ge raw_ihr[1] then begin
                print,'height range pp.hr',pp.hr,' should be between',first(raw_height),last(raw_height)
                break
            endif
            spcs_plt = un_gorge(hdr,parr_data,itime=pp.itr,irg=irg,tag=tag_sp)
            if not finite(spcs_plt[0,0]) then break
            nfftme=hdr.nfft-1
            if finite(pp.vrange[0]) then begin
                vlra=pp.vrange
                fullvlra=[-0.5,.5]*hdr.vuar
                iaspc=round(nfftme*(vlra[0]-fullvlra[0])/(fullvlra[1]-fullvlra[0]))
                ibspc=round(nfftme*(vlra[1]-fullvlra[0])/(fullvlra[1]-fullvlra[0]))
            endif else begin
                vlra=[-0.5,.5]*hdr.vuar
                iaspc=0 & ibspc=nfftme
            endelse
            timesx=unix_times[pp.itr]-unix_times[pp.itr[0]]
            mimage, db(spcs_plt[toord[iaspc:ibspc],*]), vlra, timesx , pp.srange, $
                zcolor, startcolor,pp.badcol,charsize=pp.charsize,thick=pp.thick, $
                title='Sequence of Spectra ' + unixtime2bdate(unix_times[itime], nice=0,utc=1) + ' rg:' + string(irg) , $
                ztitle='Intensity',xtitle='Velocity [m/s]',$
                heitit='Time [s]'
            if pp.raute then begin
                tag_velo = 'v'+strmid(tag_sp,2,1)
                vr_plt = syspar.dwd_me * un_gorge(hdr,parr_data,itime=pp.itr,irg=irg,tag=tag_velo)
                if n_elements(vr_plt) gt 1 then begin
                    timesx=unix_times[pp.itr[0]:pp.itr[1]]-unix_times[pp.itr[0]]
                    oplot, vr_plt ,  timesx + 99999.*(abs(vr_plt) gt 100.),color=0, max=90000.,$
                        thick = pp.thick, psym=4
                endif
            endif
        end
        cmd eq 'petenko' and arg[narg-1] ne 'off': begin
            limit_itr2contiguous,hours,max_ave,pp
            hdr = hdrs[first(where(pp.itr[0] lt ztimes))] ; pick the header of the file from which itime was loaded
            mom_height = findgen(hdr.mom_zrg) * hdr.delta_h + hdr.mom_h_min
            mom_ihr=[first(where(mom_height ge pp.hr[0])),last(where(mom_height le pp.hr[1]))]
            petenko_rf = un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag='p1')
            petenko_ve = un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag='v1')
            petenko_wd = un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag='r1')
            if finite(petenko_rf[0,0]) and n_elements(petenko_rf) gt 1 then begin 
                petenko_rf = db(petenko_rf)
                for iti=pp.itr[0],pp.itr[1] do begin
                    iti_prof = iti - pp.itr[0]
                    ix_pb = where(petenko_rf[*,iti_prof] lt pp.prange[0],nix_pb) 
                    petenko_rf[0,iti_prof] = dbz(petenko_rf[*,iti_prof], hdr, txn_power[iti],rg_offs=mom_ihr[0])
                    if nix_pb gt 0 then petenko_rf[ix_pb,iti_prof] =   !values.f_nan
                endfor
                petenko_good = petenko_filter(transpose(petenko_rf),transpose(petenko_ve), transpose(petenko_wd))
            endif
        end
        cmd eq 'petenko' and arg[narg-1] eq 'off': begin
            petenko_good = 0
        end
        cmd eq 'trc' or cmd eq 'thc': begin
            ; time range cross section
            last_draw_arg = arg
            limit_itr2contiguous,hours,max_ave,pp
            if narg ge 2 then tag_mom=arg[1] else tag_mom='p1'
            hdr = hdrs[first(where(pp.itr[0] lt ztimes))] ; pick the header of the file from which itime was loaded
            mom_height = findgen(hdr.mom_zrg) * hdr.delta_h + hdr.mom_h_min
            mom_ihr=[first(where(mom_height ge pp.hr[0])),last(where(mom_height le pp.hr[1]))]
            case strupcase(strmid(tag_mom,0,1)) of 
              'P': begin
                ztitle='SNR [dB]'
                p_tmp = un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag=tag_mom)
                if finite(p_tmp[0,0]) and n_elements(p_tmp) gt 1 then prof=db(p_tmp) else prof = p_tmp
                zrange = pp.prange
              end
              'Z': begin
                ztitle='dBZ [dB]'
                chan_str = strmid(tag_mom,1) & chan=fix(chan_str)-1
                tag_mom = 'P'+chan_str
                p_tmp = un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag=tag_mom)
                if finite(p_tmp[0,0]) and n_elements(p_tmp) gt 1 then begin
                    prof = db(p_tmp) 
                    good = prof ge pp.prange[0]
                    for iti=pp.itr[0],pp.itr[1] do begin
                        iti_prof = iti - pp.itr[0]
                        if kp.calibr[0] gt -1000 and cpw[iti,chan] gt 0 then begin
                            calibr = npw[iti,chan]/cpw[iti,chan] * linear(kp.calibr[chan])  
                        endif else calibr=1.
                        prof[0,iti_prof] = dbz(prof[*,iti_prof]*calibr, hdr, txn_power[iti],rg_offs=mom_ihr[0]) $
                            + 9999. * (good[*,iti_prof] eq 0)
                    endfor
                endif else prof = p_tmp
                zrange = pp.zrange
              end
              'V': begin
                ztitle='Velocity [m/s]'
                prof =  syspar.dwd_me * un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag=tag_mom) 
                if finite(pp.vrange[0]) then zrange=pp.vrange else zrange=[-0.5,.5]*hdr.vuar
              end
              'R': begin
                ztitle='Peak Width [m/s]'
                prof =  un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag=tag_mom) 
                zrange = pp.rrange 
              end
              'L': begin
                if strupcase(tag_mom) eq 'LDR' then begin
                  ztitle='LDR [dB]'
                  p1 = db(un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag='p1'))
                  p2 = db(un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag='p2'))
                  good = p1 ge pp.prange[0] and p2 ge pp.prange[0]
                  if n_elements(p1) gt 1 and n_elements(p2) gt 1 then begin
                      prof = (p2 - p1) * good + 100.* (good eq 0)
                  endif else prof = !values.f_nan
                  zrange = pp.lrange
                endif else prof = !values.f_nan
              end
              else: begin
                  prof = !values.f_nan
                  zrange=[0,1]
              end
            endcase

            if finite(prof[0,0]) then begin
                good_times = txn_power[pp.itr[0]:pp.itr[1]] ge pp.min_txn_pow $
                    and ( rec_err[pp.itr[0]:pp.itr[1]] and pp.rec_err_bad_bits) eq 0 $
                    and ( rec_err[pp.itr[0]:pp.itr[1]] and pp.rec_err_good_bits) eq pp.rec_err_good_bits $
                    and npw[pp.itr[0]:pp.itr[1],0] ge pp.min_npw
                zrg = n_elements(prof[*,0])                          ; should be mom_ihr[1]-mom_ihr[0]+1
                if pp.needrms then begin
                    chan_str = strmid(tag_mom,1)
                    if chan_str ne '1' and chan_str ne '2' then chan_str='1'
                    rms_good = un_gorge(hdr,parr_data,itime=pp.itr,irg=mom_ihr,tag='r'+chan_str) lt pp.rrange[1]
                    prof = prof * rms_good - 9999.  * (rms_good eq 0)
                endif
                
                prof = transpose(   prof +  replicate(1.,zrg) # ( (good_times ne 1) * 9999.)    ) 
                if n_elements(petenko_good) eq n_elements(prof) then begin
                    prof = prof * petenko_good - 9999. * (petenko_good eq 0)
                endif
                sxtf=!x.TICKFORMAT & !x.TICKFORMAT='LABEL_DATE'
                sxtu=!X.TICKUNITS[0] & !X.TICKUNITS[0]=pp.xtu
                dummy = LABEL_DATE(DATE_FORMAT='%h:%i')
                dara_title =     + unixtime2bdate(unix_times[pp.itr[0]], nice=2,utc=1) $ 
                    +  ' - '     + unixtime2bdate(unix_times[pp.itr[1]], nice=2,utc=1)
                mimage, prof, juldate[pp.itr], mom_height[mom_ihr]/td , zrange, $
                    zcolor, startcolor,pp.badcol,charsize=pp.charsize,thick=pp.thick, $
                    ztitle=pp.ti+ztitle,xtitle=pp.ti+'Time UT', $
                    heitit=pp.ti+'Range [km]'
                xyouts,0.49,0.93,pp.ti+pp.b4tit+ztitle+"   "+dara_title,$
                    /noclip,alignment=0.5,/normal,chars=pp.charsize
                !x.TICKFORMAT = sxtf & !X.TICKUNITS[0]=sxtu
            endif else print,'No data for plotting ' + tag_mom
        end
        cmd eq 'histo': begin
            histo=histogram(prof,min=zrange[0],max=zrange[1],nbins=pp.nbin)
            dzra = zrange[1] - zrange[0]
            x=findgen(pp.nbin)/float(pp.nbin)*dzra+zrange[0]
            plot,x,histo,xrange=zrange,xst=1,ytit='Frequency Distribution / '+string(dzra/pp.nbin,format='(g7.1)'),$
                xtit=ztitle,ylog=1,yrange=[1,max(histo)]
        end
        cmd eq 'sequence' or cmd eq 'zr': begin
            last_draw_arg = arg
            arg[0]='&'
            IxPlt = where(arg eq '&' or arg eq '|', zPltO) ; plots and oplots
            dummy = where(arg eq '&', zPlt) ; only plots that need a new frame
            multi_save=!p.multi
            erase
            iPlt=0
            for iPltO=0,zPltO-1 do begin
                ; Default Values for the plotting parameters:
                kpzr = { $
                    ra:[0.0,-1.0],$
                    ti:'',$
                    os:0.0,$
                    tr:[0d0,-1d0],$
                    co:!p.color $
                }
                ; Find out what to plot:
                plt_item = arg[IxPlt[iPltO] + 1]
                rg = (stregex(plt_item,'^h([0-9]*)$',/subexp,/extract,/fold))[1]
                srvi = (stregex(plt_item,'^srvi\.(.*)$',/subexp,/extract,/fold))[1]
                LogItemNr = (stregex(plt_item,'^l([0-9]*)$',/subexp,/extract,/fold))[1]
                ZrIsThere = 0
                case 1 of
                    rg ne '': begin
                        if (n_elements(prof) gt 1) then begin
                            ZrData = prof[*,long(rg)]
                            ZrTime = juldate[pp.itr[0]:pp.itr[1]]
                            ZrTrange = juldate[pp.itr]
                            ZrHight = long(rg) * hdr.delta_h + hdr.mom_h_min
                            kpzr.ti = ztitle + ' ' + string(ZrHight/1000.,format='(f6.2)') + ' km'
                            kpzr.ra = zrange
                            ZrIsThere = 1
                        endif
                    end
                    srvi ne '':begin
                        tagidx = where(tag_names(*parr_srvinfo[0]) eq srvi,ntagidx)
                        if ntagidx eq 1 then begin
                            ztr=pp.itr[1]-pp.itr[0]+1
                            ZrData = fltarr(ztr)
                            for iirg=0,ztr-1 do ZrData[iirg] = (*parr_srvinfo[pp.itr[0]+iirg]).(tagidx)
                            ZrTime = juldate[pp.itr[0]:pp.itr[1]]
                            ZrTrange = juldate[pp.itr]
                            kpzr.ti = 'srvinfo1.'+srvi
                            kpzr.ra = [0,1000.]
                            ZrIsThere = 1
                        endif
                    end
                    else: begin
                        if ( n_elements(log_params) gt 0 ) then begin
                            if (LogItemNr ne '') then begin
                                ZrDataIx = long(LogItemNr)
                            endif else begin
                                ZrDataIx = (where(log_params eq plt_item))[0]
                            endelse
                            if (ZrDataIx ge 0) then begin
                                ZrData = log_data.(ZrDataIx+1)
                                ZrTime = log_time
                                ZrTrange = [log_time[0],log_time[log_n-1]]
                                kpzr.ti = log_params[ZrDataIx]
                                kpzr.ra = [0.,-1.]
                                ZrIsThere = 1
                            endif
                        endif  
                    end
                endcase
                
                ; Overwrite the default plotting parameters kpzr by the settings supplied in the cmd line
                kpzr0 = IxPlt[iPltO] + 2          ; index at which the key word parameters in arg start
                if iPltO lt zPltO-1 then begin
                    kpzr1 = IxPlt[iPltO + 1]-1
                endif else begin
                    kpzr1 = narg -1
                endelse
                nkpzr = kpzr1 - kpzr0+1
                if nkpzr gt 0 then begin
                    argv2stru,arg[kpzr0:kpzr1],kpzr
                endif
                
                ; make the plot
                if ZrIsThere then begin
                    if iPlt eq 0 then begin
                        ; select the timerange in the first plot so that all plots will have the same time range
                        if kpzr.tr[0] lt kpzr.tr[1] then begin
                            ZrTrange4all = kpzr.tr
                        endif else begin
                            ZrTrange4all = ZrTrange
                        endelse
                    endif 
                    if kpzr.tr[0] lt kpzr.tr[1] then ZrTrangeX = kpzr.tr else ZrTrangeX = ZrTrange4all
                    ZrData += kpzr.os
                    if arg[IxPlt[iPltO]] eq '&' then begin
                        ; make a new frame
                        !p.multi=[++iPlt,1,zPlt]
                        if kpzr.ra[0] gt kpzr.ra[1] then begin
                            kpzr.ra = autorange(ZrData,98.,8.) ; the range is chosen to cover 98 percent of the data and 
                                                               ; afterwards it is increased by 4 percent to each side.
                        endif
                        sxtf=!X.TICKFORMAT & !X.TICKFORMAT='LABEL_DATE'
                        sxtu=!X.TICKUNITS[0] & !X.TICKUNITS[0]=pp.xtu
                        plot,ZrTime,ZrData,/nodata,yrange=kpzr.ra,ytit=kpzr.ti,xrange=ZrTrangeX,xst=1,yst=1,charsi=pp.charsize
                        !x.TICKFORMAT = sxtf & !X.TICKUNITS[0]=sxtu
                        last_ra = kpzr.ra
                    endif else kpzr.ra = last_ra
                    oplot,ZrTime,ZrData,color=kpzr.co,min=kpzr.ra[0],max=kpzr.ra[1]
                    oplot,ZrTime,ZrData,color=kpzr.co,psym=pp.zrsym
                endif
            endfor
            !p.multi=multi_save
        end
        cmd eq 'grep_log' or cmd eq 'gl': begin
            nparam = (where(arg[1:narg-1] eq ':'))[0]
            if nparam gt 0 then begin
                log_params = arg[1:nparam]
                spawn,'grep_log '+tota(arg[1:narg-1]+' ')+'> /tmp/grep_log.tmp'
                log_data = read_ascii('/tmp/grep_log.tmp', TEMPLATE={ VERSION:1.0, DATASTART:0, $
                    DELIMITER:32b, MISSINGVALUE:9e29, COMMENTSYMBOL:'#', FIELDCOUNT:nparam+1,$
                    FIELDTYPES:[3,replicate(4,nparam)], FIELDNAMES:['time',log_params],$
                    FIELDLOCATIONS:replicate(-1,nparam+1), FIELDGROUPS:lindgen(nparam+1)})
                spawn,'rm /tmp/grep_log.tmp'
                ;log_time = log_data.time/36d2 - first_day_0000
                log_time = unixtime2jul(log_data.time)
                log_n = n_elements(log_time)
                ; log_data.data[(where(log_params eq param]))[0],*)
            endif else print,'Pls. use a detached colon to separate parameters from log file names'
        end
        cmd eq '+' or cmd eq '-': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            if cmd eq '-' then stride *= -1
            itime = ((itime + stride) < (ztime-1) ) > 0
        end
        cmd eq 'u' or cmd eq 'd': begin
           if narg ge 2 then stride = long(arg[1]) else stride = 1L
           if cmd eq 'd' then stride *= -1
           irg = irg + stride
        end
        cmd eq '++' or cmd eq '--': begin
           if cmd eq '++' then stride=1L else stride=-1L
           if  ipcs eq 0 then pp.itr = [0,ztimes[ipcs]-1] $
                         else pp.itr = [ztimes[ipcs-1],ztimes[ipcs]-1]
           ipcs=(ipcs+stride+zheaders) mod zheaders
           if cmd eq '++' then itime = pp.itr[0] else itime = pp.itr[1]
        end
        cmd eq 'tr': begin
            case narg of
              1: pp.itr = [0,ztime-1]
              2: begin
                if strupcase(arg[1]) ne 'NAN' then begin
                    if double(arg[1]) ge 0d0 then begin
                        dumy=min(abs(hours-double(arg[1])),itr0)
                        pp.itr[0] = itr0
                    endif else begin
                        dumy=min(abs(hours-hours[ztime-1]-double(arg[1])),itr0)
                        pp.itr = [itr0,ztime-1]
                    endelse
                endif
              end
              3: begin
                if strupcase(arg[1]) ne 'NAN' then begin
                    dumy=min(abs(hours-double(arg[1])),itr0)
                    pp.itr[0] = itr0
                endif
                if strupcase(arg[2]) ne 'NAN' then begin
                    dumy=min(abs(hours-double(arg[2])),itr1)
                    pp.itr[1] = itr1
                endif
                if pp.itr[0] ge  pp.itr[1] then begin
                    print,'Timerange crossed over. Resetting time range'
                    pp.itr = [0,ztime-1]
                endif
              end
              else: print,'wrong number of arguments'
            endcase
        end
        cmd eq 'postscript' or cmd eq 'ps' or cmd eq 'print': begin
            if narg eq 2 then psfile = arg[1] else psfile="viewpds.ps"
            if (startup_device ne 'PS') then set_plot,'PS'
            device,/color,bits_per_pixel=8,filename=psfile ; ,yoffset= 27,xoffset=0.7
        end
        cmd eq 'png': begin
            if narg ge 2 then png_fn = arg[1] else begin
                png_fn = infile + '_' + tag_mom + '.png'
            endelse
            screen_dump = TVRD(true=1)
            WRITE_PNG, png_fn, screen_dump
        end
        cmd eq 'close':begin
            device,/close
            if startup_device ne 'PS' then set_plot,'X'
        end
        cmd eq 'set': begin
            if narg ge 2 then set_tag_name = arg[1] else set_tag_name='HELP'
            if strupcase(set_tag_name) eq 'HELP' then begin
                pp_tags = tag_names(pp)
                pph_tags = tag_names(pph)
                for i=0,n_elements(pp_tags)-1 do begin
                    helptext_tag_number = (where(pph_tags eq pp_tags[i]))[0]
                    helptext = helptext_tag_number ge 0 ? pph.(helptext_tag_number) : ''
                    print_table,pp_tags[i]+':',helptext,tabw=20,linew=79
                endfor
            endif else begin
                set_tag_number = (where(strupcase(set_tag_name) eq tag_names(pp)))[0]
                if set_tag_number ge 0 then begin
                    if narg eq 2 then print,set_tag_name+' :',pp.(set_tag_number) else begin
                        for i=0,(narg-3) < (n_elements(pp.(set_tag_number))-1) do begin
                            if strupcase(arg[i+2]) ne 'SKIP' then pp.(set_tag_number)[i] = arg[i+2]
                        endfor
                    endelse
                endif else print,'unknown variable name'
            endelse
        end
        cmd eq 'info': begin
            help,hdrs[first(where(itime lt ztimes))],/struct
            if narg eq 2 then begin
                help,ppars[first(where(itime lt ztimes))],/struct
            endif else print,"get more: info -d"
        end
        cmd eq 'stop': if lmgr(/vm) then goto,ende else stop
        cmd eq 'x' or cmd eq 'quit' or cmd eq 'exit': goto,ende
        else: begin
            setcmd = ''
            for i=0 , narg - 1 do setcmd = setcmd + arg[i] + ' '
            if lmgr(/vm) then print,'VM license can not execute '+setcmd $
            else dummi=execute( setcmd )
        end
    endcase
endwhile

ende:
if !d.name eq "X" then wdelete
heap_free,parr_data
heap_free,parr_srvinfo
exit
end
